# -*- coding: utf-8 -*-
"""miscellaneous searches"""

from __future__ import unicode_literals

from django.core.urlresolvers import reverse

from model_mommy import mommy

from balafon.Crm import models
from balafon.Search.tests import BaseTestCase


class SameAsEmailTest(BaseTestCase):
    """Search same-as"""

    def test_search_same_as_not_allowed1(self):
        """same as not allowed: search on entity group"""

        contact1 = mommy.make(
            models.Contact, lastname="ABCD", email="contact1@email1.fr", main_contact=True, has_left=False
        )
        contact1.entity.name = 'Tiny Corp'
        contact1.entity.default_contact.delete()
        contact1.entity.save()

        contact2 = mommy.make(
            models.Contact, lastname="ABCD", email="contact2@email2.fr", main_contact=True, has_left=False
        )
        contact2.entity.name = 'Other Corp'
        contact2.entity.default_contact.delete()
        contact2.entity.save()

        group = mommy.make(models.Group, name="GROUP1")
        group.entities.add(contact1.entity)
        group.entities.add(contact2.entity)
        group.save()

        contact1.same_as = models.SameAs.objects.create()
        contact1.same_as_priority = 1
        contact1.save()
        contact2.same_as = contact1.same_as
        contact2.same_as_priority = 2
        contact2.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group.id, "gr0-_-no_same_as-_-1": '0'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, contact1.email)
        self.assertNotContains(response, contact2.email)

    def test_search_same_as_not_allowed2(self):
        """same as not allowed"""
        contact1 = mommy.make(
            models.Contact, lastname="ABCD", email="contact1@email1.fr", main_contact=True, has_left=False
        )
        contact1.entity.name = 'Tiny Corp'
        contact1.entity.default_contact.delete()
        contact1.entity.save()

        contact2 = mommy.make(
            models.Contact, lastname="ABCD", email="contact2@email2.fr", main_contact=True, has_left=False
        )
        contact2.entity.name = 'Other Corp'
        contact2.entity.default_contact.delete()
        contact2.entity.save()

        group = mommy.make(models.Group, name="GROUP1")
        group.entities.add(contact1.entity)
        group.entities.add(contact2.entity)
        group.save()

        contact1.same_as = models.SameAs.objects.create()
        contact1.same_as_priority = 2
        contact1.save()
        contact2.same_as = contact1.same_as
        contact2.same_as_priority = 1
        contact2.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group.id, "gr0-_-no_same_as-_-1": '0'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)
        self.assertNotContains(response, contact1.email)
        self.assertContains(response, contact2.email)

    def test_search_same_as_not_allowed3(self):
        """same as not allowed: two groups"""
        contact1 = mommy.make(
            models.Contact, lastname="ABCD", email="contact1@email1.fr", main_contact=True, has_left=False
        )
        contact1.entity.name = 'Tiny Corp'
        contact1.entity.default_contact.delete()
        contact1.entity.save()

        contact2 = mommy.make(
            models.Contact, lastname="ABCD", email="contact2@email2.fr", main_contact=True, has_left=False
        )
        contact2.entity.name = 'Other Corp'
        contact2.entity.default_contact.delete()
        contact2.entity.save()

        group1 = mommy.make(models.Group, name="GROUP1")
        group1.entities.add(contact1.entity)
        group1.save()

        group2 = mommy.make(models.Group, name="GROUP2")
        group2.entities.add(contact2.entity)
        group2.save()

        contact1.same_as = models.SameAs.objects.create()
        contact1.same_as_priority = 2
        contact1.save()
        contact2.same_as = contact1.same_as
        contact2.same_as_priority = 1
        contact2.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group1.id, "gr0-_-no_same_as-_-1": '0', "gr1-_-group-_-0": group2.id, }

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)
        self.assertNotContains(response, contact1.email)
        self.assertContains(response, contact2.email)

    def test_search_same_as_allowed(self):
        """same as allowed"""
        contact1 = mommy.make(
            models.Contact, lastname="ABCD", email="contact1@email1.fr", main_contact=True, has_left=False
        )
        contact1.entity.name = 'Tiny Corp'
        contact1.entity.default_contact.delete()
        contact1.entity.save()

        contact2 = mommy.make(
            models.Contact, lastname="ABCD", email="contact2@email2.fr", main_contact=True, has_left=False
        )
        contact2.entity.name = 'Other Corp'
        contact2.entity.default_contact.delete()
        contact2.entity.save()

        group = mommy.make(models.Group, name="GROUP1")
        group.entities.add(contact1.entity)
        group.entities.add(contact2.entity)
        group.save()

        contact1.same_as = models.SameAs.objects.create()
        contact1.same_as_priority = 1
        contact1.save()
        contact2.same_as = contact1.same_as
        contact2.same_as_priority = 2
        contact2.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group.id, "gr0-_-no_same_as-_-1": '1'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, contact1.email)
        self.assertContains(response, contact2.email)

    def test_search_same_one_not_in_results(self):
        """
        The same-as contact without priority should be display if the same-as with priority is not returned by search
        """
        contact1 = mommy.make(
            models.Contact, lastname="ABCD", email="contact1@email1.fr", main_contact=True, has_left=False
        )
        contact1.entity.name = 'Tiny Corp'
        contact1.entity.default_contact.delete()
        contact1.entity.save()

        contact2 = mommy.make(
            models.Contact, lastname="ABCD", email="contact2@email2.fr", main_contact=True, has_left=False
        )
        contact2.entity.name = 'Other Corp'
        contact2.entity.default_contact.delete()
        contact2.entity.save()

        group = mommy.make(models.Group, name="GROUP1")
        group.contacts.add(contact2)
        group.save()

        contact1.same_as = models.SameAs.objects.create()
        contact1.same_as_priority = 1
        contact1.save()
        contact2.same_as = contact1.same_as
        contact2.same_as_priority = 2
        contact2.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)
        self.assertNotContains(response, contact1.email)
        self.assertContains(response, contact2.email)

    def test_search_same_as_priority_not_in_results(self):
        """
        The same-as contact without priority should be display if the same-as with priority is not returned by search
        """
        contact1 = mommy.make(
            models.Contact, lastname="ABCD", email="contact1@email1.fr", main_contact=True, has_left=False
        )
        contact1.entity.name = 'Tiny Corp'
        contact1.entity.default_contact.delete()
        contact1.entity.save()

        contact2 = mommy.make(
            models.Contact, lastname="ABCD", email="contact2@email2.fr", main_contact=True, has_left=False
        )
        contact2.entity.name = 'Other Corp'
        contact2.entity.default_contact.delete()
        contact2.entity.save()

        contact3 = mommy.make(
            models.Contact, lastname="ABCD", email="contact3@email3.fr", main_contact=True, has_left=False
        )
        contact3.entity.name = 'Big Corp'
        contact3.entity.default_contact.delete()
        contact3.entity.save()

        group = mommy.make(models.Group, name="GROUP1")
        group.contacts.add(contact2)
        group.contacts.add(contact3)
        group.save()

        contact1.same_as = models.SameAs.objects.create()
        contact1.same_as_priority = 1
        contact1.save()

        contact2.same_as = contact1.same_as
        contact2.same_as_priority = 3
        contact2.save()

        contact3.same_as = contact1.same_as
        contact3.same_as_priority = 2
        contact3.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group.id, "gr0-_-no_same_as-_-1": '0'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, contact3.email)
        self.assertNotContains(response, contact2.email)
        self.assertNotContains(response, contact1.email)

    def test_search_same_as_priority_not_in_results2(self):
        """
        The same-as contact without priority should be display if the same-as with priority is not returned by search
        """
        contact1 = mommy.make(
            models.Contact, lastname="ABCD", email="contact1@email1.fr", main_contact=True, has_left=False
        )
        contact1.entity.name = 'Tiny Corp'
        contact1.entity.default_contact.delete()
        contact1.entity.save()

        contact2 = mommy.make(
            models.Contact, lastname="ABCD", email="contact2@email2.fr", main_contact=True, has_left=False
        )
        contact2.entity.name = 'Other Corp'
        contact2.entity.default_contact.delete()
        contact2.entity.save()

        contact3 = mommy.make(
            models.Contact, lastname="ABCD", email="contact3@email3.fr", main_contact=True, has_left=False
        )
        contact3.entity.name = 'Big Corp'
        contact3.entity.default_contact.delete()
        contact3.entity.save()

        group = mommy.make(models.Group, name="GROUP1")
        group.contacts.add(contact2)
        group.contacts.add(contact3)
        group.save()

        contact1.same_as = models.SameAs.objects.create()
        contact1.same_as_priority = 1
        contact1.save()

        contact2.same_as = contact1.same_as
        contact2.same_as_priority = 2
        contact2.save()

        contact3.same_as = contact1.same_as
        contact3.same_as_priority = 3
        contact3.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group.id, "gr0-_-no_same_as-_-1": '0'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertNotContains(response, contact3.email)
        self.assertContains(response, contact2.email)
        self.assertNotContains(response, contact1.email)

    def test_search_same_as_priority_not_in_results3(self):
        """
        The same-as contact without priority should be display if the same-as with priority is not returned by search
        """
        contact1 = mommy.make(
            models.Contact, lastname="ABCD", email="contact1@email1.fr", main_contact=True, has_left=False
        )
        contact1.entity.name = 'Tiny Corp'
        contact1.entity.default_contact.delete()
        contact1.entity.save()

        contact2 = mommy.make(
            models.Contact, lastname="ABCD", email="contact2@email2.fr", main_contact=True, has_left=False
        )
        contact2.entity.name = 'Other Corp'
        contact2.entity.default_contact.delete()
        contact2.entity.save()

        contact3 = mommy.make(
            models.Contact, lastname="ABCD", email="contact3@email3.fr", main_contact=True, has_left=False
        )
        contact3.entity.name = 'Big Corp'
        contact3.entity.default_contact.delete()
        contact3.entity.save()

        group = mommy.make(models.Group, name="GROUP1")
        group.contacts.add(contact2)
        group.save()

        contact1.same_as = models.SameAs.objects.create()
        contact1.same_as_priority = 1
        contact1.save()

        contact2.same_as = contact1.same_as
        contact2.same_as_priority = 2
        contact2.save()

        url = reverse('search')

        data = {"gr0-_-group-_-0": group.id, "gr0-_-no_same_as-_-1": '0'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, contact2.email)
        self.assertNotContains(response, contact1.email)


class SameAsNameTest(BaseTestCase):
    """Search same-as"""

    def _make_contact(self, **kwargs):
        kwargs.setdefault('has_left', False)
        kwargs.setdefault('main_contact', True)
        entity = mommy.make(models.Entity)
        contact = mommy.make(models.Contact, entity=entity, **kwargs)
        contact.entity.default_contact.delete()
        contact.entity.save()
        return contact

    def test_search_duplicated_contacts(self):
        """same as not allowed: search on entity group"""

        contact1 = self._make_contact(firstname="Pierre", lastname="Dupond", email="a@me.fr")
        contact2 = self._make_contact(firstname="Paul", lastname="Dupond", email="b@me.fr")
        contact3 = self._make_contact(firstname="Pierre", lastname="Dupond", email="c@me.fr")
        contact4 = self._make_contact(firstname="Pierre", lastname="Dupont", email="d@me.fr")

        url = reverse('search')

        data = {"gr0-_-duplicated_contacts-_-1": '1'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, contact1.email)
        self.assertNotContains(response, contact2.email)
        self.assertContains(response, contact3.email)
        self.assertNotContains(response, contact4.email)

    def test_search_several_duplicated_contacts(self):
        """same as not allowed: search on entity group"""

        contact1 = self._make_contact(firstname="Pierre", lastname="Dupond", email="a@me.fr")
        contact2 = self._make_contact(firstname="Paul", lastname="Dupond", email="b@me.fr")
        contact3 = self._make_contact(firstname="Pierre", lastname="Dupond", email="c@me.fr")
        contact4 = self._make_contact(firstname="Pierre", lastname="Dupont", email="d@me.fr")
        contact5 = self._make_contact(firstname="Pierre", lastname="Dupond", email="e@me.fr")
        contact6 = self._make_contact(firstname="Pierre", lastname="Dupond", email="f@me.fr")

        url = reverse('search')

        data = {"gr0-_-duplicated_contacts-_-1": '1'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, contact1.email)
        self.assertNotContains(response, contact2.email)
        self.assertContains(response, contact3.email)
        self.assertNotContains(response, contact4.email)
        self.assertContains(response, contact5.email)
        self.assertContains(response, contact6.email)

    def test_search_duplicated_contacts_name_only(self):
        """same as not allowed: search on entity group"""

        contact1 = self._make_contact(firstname="Pierre", lastname="Dupond", email="a@me.fr")
        contact2 = self._make_contact(firstname="Paul", lastname="Dupond", email="b@me.fr")
        contact3 = self._make_contact(firstname="Pierre", lastname="Dupond", email="c@me.fr")
        contact4 = self._make_contact(firstname="Pierre", lastname="Dupont", email="d@me.fr")

        url = reverse('search')

        data = {"gr0-_-duplicated_contacts-_-1": '2'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, contact1.email)
        self.assertContains(response, contact2.email)
        self.assertContains(response, contact3.email)
        self.assertNotContains(response, contact4.email)

    def test_search_ignore_unknown(self):
        """same as not allowed: search on entity group"""

        contact1 = self._make_contact(firstname="", lastname="", email="a@me.fr")
        contact2 = self._make_contact(firstname="", lastname="", email="b@me.fr")

        url = reverse('search')

        data = {"gr0-_-duplicated_contacts-_-1": '1'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertNotContains(response, contact1.email)
        self.assertNotContains(response, contact2.email)

    def test_search_ignore_unknown_name_only(self):
        """same as not allowed: search on entity group"""

        contact1 = self._make_contact(firstname="A", lastname="", email="a@me.fr")
        contact2 = self._make_contact(firstname="B", lastname="", email="b@me.fr")

        url = reverse('search')

        data = {"gr0-_-duplicated_contacts-_-1": '2'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertNotContains(response, contact1.email)
        self.assertNotContains(response, contact2.email)




# -*- coding: utf-8 -*-
"""
app configuration
"""

from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from django.apps import AppConfig


class BalafonAppConfig(AppConfig):
    name = 'balafon.Crm'
    verbose_name = _(u"Balafon CRM")



# -*- coding: utf-8 -*-
"""unit testing"""

# pylint: disable=too-many-locals
# pylint: disable=too-many-statements
# pylint: disable=too-many-branches

from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.core.urlresolvers import reverse

from coop_cms.tests import BeautifulSoup
from model_mommy import mommy

from balafon.unit_tests import TestCase
from balafon.Crm import models
from balafon.Crm.tests import BaseTestCase


class DoActionTest(BaseTestCase):
    """Do action"""

    def test_do_action(self):
        """do it"""
        action = mommy.make(models.Action, done=False)
        response = self.client.get(reverse('crm_do_action', args=[action.id]))
        self.assertEqual(200, response.status_code)
        self.assertEqual(action.done, False)

        response = self.client.post(reverse('crm_do_action', args=[action.id]), data={'done': True})
        self.assertEqual(200, response.status_code)
        self.assertContains(response, reverse("crm_board_panel"))
        action = models.Action.objects.get(id=action.id)
        self.assertEqual(action.done, True)

    def test_undo_action(self):
        """undo it"""
        action = mommy.make(models.Action, done=True)
        response = self.client.get(reverse('crm_do_action', args=[action.id]))
        self.assertEqual(200, response.status_code)
        self.assertEqual(action.done, True)

        response = self.client.post(reverse('crm_do_action', args=[action.id]), data={'done': False})
        self.assertEqual(200, response.status_code)
        self.assertContains(response, reverse("crm_board_panel"))
        action = models.Action.objects.get(id=action.id)
        self.assertEqual(action.done, False)

    def test_view_action_warning_not_in_charge(self):
        """warning if not in charge"""
        user = mommy.make(User, is_active=True, is_staff=True, last_name="L", first_name="F")
        team_member = mommy.make(models.TeamMember, user=user)
        action = mommy.make(models.Action, done=True, in_charge=team_member)
        response = self.client.get(reverse('crm_do_action', args=[action.id]))
        self.assertEqual(200, response.status_code)
        self.assertContains(response, "warning")


class ActionAutoGenerateNumberTestCase(TestCase):
    """test number can be autogenerated"""

    def test_create_auto_number(self):
        """create auto number"""
        action_type = mommy.make(models.ActionType, last_number=0, number_auto_generated=True)
        action = models.Action.objects.create(type=action_type, subject="a")
        self.assertEqual(action.number, 1)

    def test_create_several_auto_number(self):
        """create several numbers"""
        action_type = mommy.make(models.ActionType, last_number=0, number_auto_generated=True)
        action = models.Action.objects.create(type=action_type, subject="a")
        self.assertEqual(action.number, 1)
        action = models.Action.objects.create(type=action_type, subject="a")
        self.assertEqual(action.number, 2)
        no_gen_type = mommy.make(models.ActionType, last_number=0, number_auto_generated=False)
        action = models.Action.objects.create(type=no_gen_type, subject="a")
        self.assertEqual(action.number, 0)
        action = models.Action.objects.create(type=action_type, subject="a")
        self.assertEqual(action.number, 3)

    def test_save_several_auto_number(self):
        """save : number is not changed"""
        action_type = mommy.make(models.ActionType, last_number=0, number_auto_generated=True)
        action = models.Action.objects.create(type=action_type, subject="a")
        self.assertEqual(action.number, 1)
        action.save()
        self.assertEqual(action.number, 1)

    def test_create_several_auto_types(self):
        """number generated by type"""
        action_type = mommy.make(models.ActionType, last_number=0, number_auto_generated=True)
        action = models.Action.objects.create(type=action_type, subject="a")
        self.assertEqual(action.number, 1)
        action = models.Action.objects.create(type=action_type, subject="a")
        self.assertEqual(action.number, 2)
        no_gen_type = mommy.make(models.ActionType, last_number=27, number_auto_generated=True)
        action = models.Action.objects.create(type=no_gen_type, subject="a")
        self.assertEqual(action.number, 28)
        action = models.Action.objects.create(type=action_type, subject="a")
        self.assertEqual(action.number, 3)

    def test_create_no_number_generation(self):
        """create action with no auto generation"""
        action_type = mommy.make(models.ActionType, last_number=0, number_auto_generated=False)
        action = models.Action.objects.create(type=action_type, subject="a")
        self.assertEqual(action.number, 0)


class ActionMenuTest(BaseTestCase):
    """Custom menu action"""

    def test_action_menu(self):
        """test custom menus are displayed"""
        action_type1 = mommy.make(models.ActionType)

        action = mommy.make(models.Action, type=action_type1)
        entity = mommy.make(models.Entity)
        action.entities.add(entity)
        action.save()

        menu1 = mommy.make(
            models.ActionMenu, action_type=action_type1,
            icon="cog", label="DO ME", view_name='crm_do_action', a_attrs='class="colorbox-form"', order_index=2
        )

        menu2 = mommy.make(
            models.ActionMenu, action_type=action_type1,
            icon="remove", label="DELETE ME", view_name='crm_delete_action', a_attrs='target="_blank"', order_index=1
        )

        url = reverse('crm_view_entity', args=[entity.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)

        li_tags = soup.select('.ut-custom-action-menu-item')
        self.assertEqual(len(li_tags), 2)

        self.assertEqual(li_tags[0].text.strip(), menu2.label)
        self.assertEqual(li_tags[0].a['target'], '_blank')
        self.assertEqual(li_tags[0].a['href'], reverse('crm_delete_action', args=[action.id]))
        self.assertEqual(li_tags[0].a.span['class'], ['glyphicon', 'glyphicon-{0}'.format(menu2.icon)])

        self.assertEqual(li_tags[1].text.strip(), menu1.label)
        self.assertEqual(li_tags[1].a['class'], ['colorbox-form'])
        self.assertEqual(li_tags[1].a['href'], reverse('crm_do_action', args=[action.id]))
        self.assertEqual(li_tags[1].a.span['class'], ['glyphicon', 'glyphicon-{0}'.format(menu1.icon)])

    def test_action_menu_status(self):
        """test custom menus are displayed"""
        action_type1 = mommy.make(models.ActionType)
        action_status1 = mommy.make(models.ActionStatus)
        action_status2 = mommy.make(models.ActionStatus)
        action_type1.allowed_status.add(action_status1)
        action_type1.allowed_status.add(action_status2)
        action_type1.save()

        action = mommy.make(models.Action, type=action_type1, status=action_status1)
        entity = mommy.make(models.Entity)
        action.entities.add(entity)
        action.save()

        menu1 = mommy.make(
            models.ActionMenu, action_type=action_type1,
            icon="cog", label="DO ME", view_name='crm_do_action', a_attrs='class="colorbox-form"', order_index=2
        )
        menu1.only_for_status.add(action_status1)
        menu1.save()

        menu2 = mommy.make(
            models.ActionMenu, action_type=action_type1,
            icon="remove", label="DELETE ME", view_name='crm_delete_action', a_attrs='target="_blank"', order_index=1
        )
        menu2.only_for_status.add(action_status2)
        menu2.save()

        url = reverse('crm_view_entity', args=[entity.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)

        li_tags = soup.select('.ut-custom-action-menu-item')
        self.assertEqual(len(li_tags), 1)

        self.assertEqual(li_tags[0].text.strip(), menu1.label)
        self.assertEqual(li_tags[0].a['class'], ['colorbox-form'])
        self.assertEqual(li_tags[0].a['href'], reverse('crm_do_action', args=[action.id]))
        self.assertEqual(li_tags[0].a.span['class'], ['glyphicon', 'glyphicon-{0}'.format(menu1.icon)])

    def test_action_menu_status_ony_empty(self):
        """test custom menus are displayed"""
        action_type1 = mommy.make(models.ActionType)
        action_status1 = mommy.make(models.ActionStatus)
        action_status2 = mommy.make(models.ActionStatus)
        action_type1.allowed_status.add(action_status1)
        action_type1.allowed_status.add(action_status2)
        action_type1.save()

        action = mommy.make(models.Action, type=action_type1, status=action_status1)
        entity = mommy.make(models.Entity)
        action.entities.add(entity)
        action.save()

        menu1 = mommy.make(
            models.ActionMenu, action_type=action_type1,
            icon="cog", label="DO ME", view_name='crm_do_action', a_attrs='class="colorbox-form"', order_index=2
        )

        menu2 = mommy.make(
            models.ActionMenu, action_type=action_type1,
            icon="remove", label="DELETE ME", view_name='crm_delete_action', a_attrs='target="_blank"', order_index=1
        )
        menu2.only_for_status.add(action_status2)
        menu2.save()

        url = reverse('crm_view_entity', args=[entity.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)

        li_tags = soup.select('.ut-custom-action-menu-item')
        self.assertEqual(len(li_tags), 1)

        self.assertEqual(li_tags[0].text.strip(), menu1.label)
        self.assertEqual(li_tags[0].a['class'], ['colorbox-form'])
        self.assertEqual(li_tags[0].a['href'], reverse('crm_do_action', args=[action.id]))
        self.assertEqual(li_tags[0].a.span['class'], ['glyphicon', 'glyphicon-{0}'.format(menu1.icon)])

    def test_action_menu_one_not_set(self):
        """test custom menus are displayed only for associated types"""
        action_type1 = mommy.make(models.ActionType)
        action_type2 = mommy.make(models.ActionType)

        action = mommy.make(models.Action, type=action_type1)
        entity = mommy.make(models.Entity)
        action.entities.add(entity)
        action.save()

        menu1 = mommy.make(
            models.ActionMenu, action_type=action_type1,
            icon="cog", label="DO ME", view_name='crm_do_action', a_attrs='class="colorbox-form"', order_index=2
        )

        mommy.make(
            models.ActionMenu, action_type=action_type2,
            icon="remove", label="DELETE ME", view_name='crm_delete_action', a_attrs='target="_blank"', order_index=1
        )

        url = reverse('crm_view_entity', args=[entity.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)

        li_tags = soup.select('.ut-custom-action-menu-item')
        self.assertEqual(len(li_tags), 1)

        self.assertEqual(li_tags[0].text.strip(), menu1.label)
        self.assertEqual(li_tags[0].a['class'], ['colorbox-form'])
        self.assertEqual(li_tags[0].a['href'], reverse('crm_do_action', args=[action.id]))
        self.assertEqual(li_tags[0].a.span['class'], ['glyphicon', 'glyphicon-{0}'.format(menu1.icon)])

    def test_action_menu_no_type(self):
        """test action is displayed if no type defined"""
        action = mommy.make(models.Action, type=None)
        entity = mommy.make(models.Entity)
        action.entities.add(entity)
        action.save()

        url = reverse('crm_view_entity', args=[entity.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)

        li_tags = soup.select('.ut-custom-action-menu-item')
        self.assertEqual(len(li_tags), 0)

    def test_action_no_menu(self):
        """test action is displayed if no custom menu"""
        action_type1 = mommy.make(models.ActionType)

        action = mommy.make(models.Action, type=action_type1)
        entity = mommy.make(models.Entity)
        action.entities.add(entity)
        action.save()

        url = reverse('crm_view_entity', args=[entity.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)

        li_tags = soup.select('.ut-custom-action-menu-item')
        self.assertEqual(len(li_tags), 0)

    def test_action_inavlid_view_name(self):
        """test custom menus are displayed only for associated types"""
        action_type1 = mommy.make(models.ActionType)

        action = mommy.make(models.Action, type=action_type1)
        entity = mommy.make(models.Entity)
        action.entities.add(entity)
        action.save()

        menu1 = mommy.make(
            models.ActionMenu, action_type=action_type1,
            icon="cog", label="DO ME", view_name='crm_blabla_dummy', a_attrs='class="colorbox-form"', order_index=2
        )

        url = reverse('crm_view_entity', args=[entity.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)

        soup = BeautifulSoup(response.content)

        li_tags = soup.select('.ut-custom-action-menu-item')
        self.assertEqual(len(li_tags), 1)

        self.assertEqual(li_tags[0].text.strip(), menu1.label)
        self.assertEqual(li_tags[0].a['class'], ['colorbox-form'])
        self.assertEqual(li_tags[0].a['href'], '')
        self.assertEqual(li_tags[0].a.span['class'], ['glyphicon', 'glyphicon-{0}'.format(menu1.icon)])


class CloneActionTest(BaseTestCase):
    """clone an action"""

    def test_view_clone_action_1_type(self):
        """it should display hidden input for the only allowed type"""
        action_type_1 = mommy.make(models.ActionType)
        action_type_2 = mommy.make(models.ActionType)
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.save()

        action = mommy.make(models.Action, type=action_type_1)

        response = self.client.get(reverse('crm_clone_action', args=[action.id]))
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        html_fields = soup.select("#id_action_type")
        self.assertEqual(1, len(html_fields))
        self.assertEqual(html_fields[0]["type"], "hidden")
        self.assertEqual(html_fields[0]["value"], "{0}".format(action_type_2.id))

    def test_view_clone_action_2_types(self):
        """it should display a select with allowed types"""
        action_type_1 = mommy.make(models.ActionType, order_index=1)
        action_type_2 = mommy.make(models.ActionType, order_index=2)
        action_type_3 = mommy.make(models.ActionType, order_index=3)
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.next_action_types.add(action_type_3)
        action_type_1.save()

        action = mommy.make(models.Action, type=action_type_1)

        response = self.client.get(reverse('crm_clone_action', args=[action.id]))
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        html_fields = soup.select("#id_action_type")
        self.assertEqual(1, len(html_fields))
        options = html_fields[0].select("option")
        self.assertEqual(len(options), 2)
        self.assertEqual(options[0]["value"], "{0}".format(action_type_2.id))
        self.assertEqual(options[1]["value"], "{0}".format(action_type_3.id))

    def test_view_clone_action_no_types(self):
        """it should display an empty select"""
        action_type_1 = mommy.make(models.ActionType, order_index=1)

        action = mommy.make(models.Action, type=action_type_1)

        response = self.client.get(reverse('crm_clone_action', args=[action.id]))
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        html_fields = soup.select("#id_action_type")
        self.assertEqual(1, len(html_fields))
        options = html_fields[0].select("option")
        self.assertEqual(len(options), 0)

    def test_view_clone_action_type_is_none(self):
        """it should display 404"""
        action = mommy.make(models.Action, type=None)
        response = self.client.get(reverse('crm_clone_action', args=[action.id]))
        self.assertEqual(404, response.status_code)

    def test_view_clone_anonymous(self):
        """it should display hidden input for the only allowed type"""
        self.client.logout()

        action_type_1 = mommy.make(models.ActionType)
        action_type_2 = mommy.make(models.ActionType)
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.save()

        action = mommy.make(models.Action, type=action_type_1)

        response = self.client.get(reverse('crm_clone_action', args=[action.id]))
        self.assertEqual(302, response.status_code)

    def test_view_clone_non_staff(self):
        """it should display hidden input for the only allowed type"""
        self.user.is_staff = False
        self.user.save()

        action_type_1 = mommy.make(models.ActionType)
        action_type_2 = mommy.make(models.ActionType)
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.save()

        action = mommy.make(models.Action, type=action_type_1)

        response = self.client.get(reverse('crm_clone_action', args=[action.id]))
        self.assertEqual(302, response.status_code)

    def test_post_clone_action(self):
        """it should clone"""
        action_status_1 = mommy.make(models.ActionStatus)
        action_type_1 = mommy.make(models.ActionType)
        action_type_1.allowed_status.add(action_status_1)
        action_type_2 = mommy.make(models.ActionType)
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.save()

        contact = mommy.make(models.Contact)
        entity = mommy.make(models.Entity)
        team_member = mommy.make(models.TeamMember)

        action = mommy.make(models.Action, type=action_type_1, done=True, status=action_status_1, in_charge=team_member)

        action.contacts.add(contact)
        action.entities.add(entity)

        action.save()

        data = {
            'action_type': action_type_2.id
        }

        response = self.client.post(
            reverse('crm_clone_action', args=[action.id]),
            data=data
        )
        self.assertEqual(200, response.status_code)

        self.assertEqual(2, models.Action.objects.count())
        original_action = models.Action.objects.get(type=action_type_1)
        new_action = models.Action.objects.get(type=action_type_2)

        self.assertContains(response, 'reload: {0}'.format(reverse('crm_edit_action', args=[new_action.id])))

        self.assertEqual(original_action.subject, new_action.subject)
        self.assertEqual(new_action.parent, original_action)
        self.assertEqual(new_action.contacts.count(), 1)
        self.assertEqual(new_action.entities.count(), 1)
        self.assertEqual(list(original_action.contacts.all()), list(new_action.contacts.all()))
        self.assertEqual(list(original_action.entities.all()), list(new_action.entities.all()))
        self.assertEqual(new_action.done, False)
        self.assertEqual(new_action.in_charge, original_action.in_charge)
        self.assertEqual(original_action.done, True)
        self.assertEqual(new_action.status, None)
        self.assertEqual(new_action.planned_date, original_action.planned_date)
        self.assertEqual(new_action.amount, original_action.amount)

    def test_post_clone_action_dont_affect(self):
        """it should clone without assigning the user to the task"""
        action_status_1 = mommy.make(models.ActionStatus)
        action_type_1 = mommy.make(models.ActionType)
        action_type_1.allowed_status.add(action_status_1)
        action_type_2 = mommy.make(models.ActionType, not_assigned_when_cloned=True)
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.save()

        contact = mommy.make(models.Contact)
        entity = mommy.make(models.Entity)

        team_member = mommy.make(models.TeamMember)

        action = mommy.make(models.Action, type=action_type_1, done=True, status=action_status_1, in_charge=team_member)

        action.contacts.add(contact)
        action.entities.add(entity)

        action.save()

        data = {
            'action_type': action_type_2.id
        }

        response = self.client.post(
            reverse('crm_clone_action', args=[action.id]),
            data=data
        )
        self.assertEqual(200, response.status_code)

        self.assertEqual(2, models.Action.objects.count())
        original_action = models.Action.objects.get(type=action_type_1)
        new_action = models.Action.objects.get(type=action_type_2)

        self.assertContains(response, 'reload: {0}'.format(reverse('crm_edit_action', args=[new_action.id])))

        self.assertEqual(original_action.subject, new_action.subject)
        self.assertEqual(new_action.parent, original_action)
        self.assertEqual(new_action.contacts.count(), 1)
        self.assertEqual(new_action.entities.count(), 1)
        self.assertEqual(list(original_action.contacts.all()), list(new_action.contacts.all()))
        self.assertEqual(list(original_action.entities.all()), list(new_action.entities.all()))
        self.assertEqual(new_action.done, False)
        self.assertEqual(original_action.done, True)
        self.assertEqual(new_action.in_charge, None)
        self.assertEqual(new_action.status, None)
        self.assertEqual(new_action.planned_date, original_action.planned_date)
        self.assertEqual(new_action.amount, original_action.amount)

    def test_post_clone_action_default_status(self):
        """it should clone"""
        action_status = mommy.make(models.ActionStatus)
        action_status_1 = mommy.make(models.ActionStatus)
        action_type_1 = mommy.make(models.ActionType)
        action_type_1.allowed_status.add(action_status_1)
        action_type_2 = mommy.make(models.ActionType, default_status=action_status)
        action_type_2.allowed_status.add(action_status)
        action_type_2.save()
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.save()

        contact = mommy.make(models.Contact)
        entity = mommy.make(models.Entity)

        action = mommy.make(models.Action, type=action_type_1, done=True, status=action_status_1)

        action.contacts.add(contact)
        action.entities.add(entity)

        action.save()

        data = {
            'action_type': action_type_2.id
        }

        response = self.client.post(
            reverse('crm_clone_action', args=[action.id]),
            data=data
        )
        self.assertEqual(200, response.status_code)

        self.assertEqual(2, models.Action.objects.count())
        original_action = models.Action.objects.get(type=action_type_1)
        new_action = models.Action.objects.get(type=action_type_2)

        self.assertContains(response, 'reload: {0}'.format(reverse('crm_edit_action', args=[new_action.id])))

        self.assertEqual(original_action.subject, new_action.subject)
        self.assertEqual(new_action.parent, original_action)
        self.assertEqual(new_action.contacts.count(), 1)
        self.assertEqual(new_action.entities.count(), 1)
        self.assertEqual(list(original_action.contacts.all()), list(new_action.contacts.all()))
        self.assertEqual(list(original_action.entities.all()), list(new_action.entities.all()))
        self.assertEqual(new_action.done, False)
        self.assertEqual(original_action.done, True)
        self.assertEqual(new_action.status, action_status)
        self.assertEqual(new_action.planned_date, original_action.planned_date)
        self.assertEqual(new_action.amount, original_action.amount)

    def test_post_clone_action_anonymous(self):
        """it should not clone and redirect to login page"""
        action_type_1 = mommy.make(models.ActionType)
        action_type_2 = mommy.make(models.ActionType)
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.save()

        action = mommy.make(models.Action, type=action_type_1)

        data = {
            'action_type': action_type_2.id
        }

        self.client.logout()

        response = self.client.post(
            reverse('crm_clone_action', args=[action.id]),
            data=data
        )
        self.assertEqual(302, response.status_code)

        self.assertEqual(1, models.Action.objects.count())

    def test_post_clone_action_non_staff(self):
        """it should not clone and redirect to login page"""
        action_type_1 = mommy.make(models.ActionType)
        action_type_2 = mommy.make(models.ActionType)
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.save()

        action = mommy.make(models.Action, type=action_type_1)

        data = {
            'action_type': action_type_2.id
        }

        self.user.is_staff = False
        self.user.save()

        response = self.client.post(
            reverse('crm_clone_action', args=[action.id]),
            data=data
        )
        self.assertEqual(302, response.status_code)

        self.assertEqual(1, models.Action.objects.count())

    def test_post_clone_action_invalid(self):
        """it should not clone and show error"""
        action_type_1 = mommy.make(models.ActionType)
        action_type_2 = mommy.make(models.ActionType)
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.save()

        action = mommy.make(models.Action, type=action_type_1)

        data = {
            'action_type': 'bla'
        }

        response = self.client.post(
            reverse('crm_clone_action', args=[action.id]),
            data=data
        )
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select(".error-msg")), 1)
        self.assertEqual(1, models.Action.objects.count())

    def test_post_clone_no_type(self):
        """it should not clone and show error"""
        action_type_1 = mommy.make(models.ActionType)
        action_type_2 = mommy.make(models.ActionType)
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.save()

        action = mommy.make(models.Action, type=action_type_1)

        data = {
            'action_type': ''
        }

        response = self.client.post(
            reverse('crm_clone_action', args=[action.id]),
            data=data
        )
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select(".error-msg")), 1)
        self.assertEqual(1, models.Action.objects.count())

    def test_post_clone_other_type(self):
        """it should not clone and show error"""
        action_type_1 = mommy.make(models.ActionType)
        action_type_2 = mommy.make(models.ActionType)
        action_type_3 = mommy.make(models.ActionType)
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.save()

        action = mommy.make(models.Action, type=action_type_1)

        data = {
            'action_type': action_type_3.id
        }

        response = self.client.post(
            reverse('crm_clone_action', args=[action.id]),
            data=data
        )
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select(".error-msg")), 1)
        self.assertEqual(1, models.Action.objects.count())

    def test_post_clone_type_is_none(self):
        """it should not clone and 404"""
        action_type_1 = mommy.make(models.ActionType)
        action_type_2 = mommy.make(models.ActionType)
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.save()

        action = mommy.make(models.Action, type=None)

        data = {
            'action_type': action_type_2,
        }

        response = self.client.post(
            reverse('crm_clone_action', args=[action.id]),
            data=data
        )
        self.assertEqual(404, response.status_code)
        self.assertEqual(1, models.Action.objects.count())

    def test_action_menu(self):
        """it should create and delete an action menu"""
        action_type_1 = mommy.make(models.ActionType)
        action_type_2 = mommy.make(models.ActionType)
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.save()

        self.assertEqual(1, models.ActionMenu.objects.count())
        menu = models.ActionMenu.objects.all()[0]
        self.assertEqual(menu.action_type, action_type_1)
        self.assertEqual(menu.view_name, 'crm_clone_action')

        action_type_1.next_action_types.clear()
        action_type_1.save()

        self.assertEqual(0, models.ActionMenu.objects.count())


class MailtoActionTest(BaseTestCase):
    """Send email to action contacts"""

    def test_mailto_action_contact(self):
        """send email to the ony contact"""

        type = mommy.make(models.ActionType)
        action = mommy.make(models.Action, subject="Test", type=type)
        contact = mommy.make(models.Contact, email="toto@toto.fr")
        action.contacts.add(contact)
        action.save()

        mommy.make(models.MailtoSettings, body_template="Hello", action_type=type)

        url = reverse("crm_mailto_action", args=[action.id])

        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Test&body=Hello")

    def test_mailto_action_entity(self):
        """send email to the ony entity"""

        type = mommy.make(models.ActionType)
        action = mommy.make(models.Action, subject="Test", type=type)
        entity = mommy.make(models.Entity, email="toto@toto.fr")
        action.entities.add(entity)
        action.save()

        mommy.make(models.MailtoSettings, body_template="Hello", action_type=type)

        url = reverse("crm_mailto_action", args=[action.id])

        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Test&body=Hello")

    def test_mailto_action_several(self):
        """send email to the several contacts"""

        type = mommy.make(models.ActionType)
        action = mommy.make(models.Action, subject="Test", type=type)

        entity = mommy.make(models.Entity, email="entity@toto.fr")

        contact1 = mommy.make(models.Contact, email="toto@toto.fr")
        contact2 = mommy.make(models.Contact, email="c2@toto.fr")
        contact3 = mommy.make(models.Contact, entity=entity, email="")
        contact4 = mommy.make(models.Contact, entity=entity, email="c3@toto.fr")
        contact5 = mommy.make(models.Contact)
        contact6 = mommy.make(models.Contact, email="toto@toto.fr")
        action.contacts.add(contact1)
        action.contacts.add(contact2)
        action.contacts.add(contact3)
        action.contacts.add(contact4)
        action.contacts.add(contact5)
        action.contacts.add(contact6)

        entity = mommy.make(models.Entity, email="blabla@toto.fr")
        action.entities.add(entity)

        action.save()

        mommy.make(models.MailtoSettings, body_template="Hello", action_type=type)

        url = reverse("crm_mailto_action", args=[action.id])

        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(
            response['Location'],
            "mailto:blabla@toto.fr,c2@toto.fr,c3@toto.fr,entity@toto.fr,toto@toto.fr?subject=Test&body=Hello"
        )

    def test_mailto_action_bcc(self):
        """send email to the ony contact"""

        type = mommy.make(models.ActionType)
        action = mommy.make(models.Action, subject="Test", type=type)

        contact = mommy.make(models.Contact, email="toto@toto.fr")
        action.contacts.add(contact)
        action.save()

        mommy.make(models.MailtoSettings, body_template="Hello", bcc=True, action_type=type)

        url = reverse("crm_mailto_action", args=[action.id])

        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], "mailto:?bcc=toto@toto.fr&subject=Test&body=Hello")

    def test_mailto_action_subject(self):
        """send email to the ony contact"""

        type = mommy.make(models.ActionType)
        action = mommy.make(models.Action, subject="Test", type=type)

        contact = mommy.make(models.Contact, email="toto@toto.fr")
        action.contacts.add(contact)
        action.save()

        mommy.make(models.MailtoSettings, body_template="Hello", subject="Subject", action_type=type)

        url = reverse("crm_mailto_action", args=[action.id])

        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Subject&body=Hello")

    def test_mailto_action_body(self):
        """send email to the ony contact"""

        type = mommy.make(models.ActionType)
        action = mommy.make(models.Action, subject="Test", type=type)

        contact = mommy.make(models.Contact, email="toto@toto.fr")
        action.contacts.add(contact)
        action.save()

        mommy.make(models.MailtoSettings, body_template="Hello {{ action.subject }}", action_type=type)

        url = reverse("crm_mailto_action", args=[action.id])

        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Test&body=Hello%20Test")

    def test_mailto_action_no_settings(self):
        """send email to the ony contact"""

        # Give super user right in order to allow access to admin changelist
        self.user.is_superuser = True
        self.user.save()

        type = mommy.make(models.ActionType)
        action = mommy.make(models.Action, subject="Test", type=type)
        contact = mommy.make(models.Contact, email="toto@toto.fr")
        action.contacts.add(contact)
        action.save()

        url = reverse("crm_mailto_action", args=[action.id])

        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Test&body=")

    def test_mailto_action_no_settings_not_superuser(self):
        """send email to the ony contact"""

        # No super user right in order to forbid access to admin changelist
        self.user.is_superuser = False
        self.user.save()

        type = mommy.make(models.ActionType)
        action = mommy.make(models.Action, subject=u"Test", type=type)
        contact = mommy.make(models.Contact, email="toto@toto.fr")
        action.contacts.add(contact)
        action.save()

        url = reverse("crm_mailto_action", args=[action.id])

        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Test&body=")

    def test_mailto_action_invalid_action(self):
        """send email to the ony contact"""

        type = mommy.make(models.ActionType)
        action = mommy.make(models.Action, subject="Test", type=type)
        contact = mommy.make(models.Contact, email="toto@toto.fr")
        action.contacts.add(contact)
        action.save()

        mommy.make(models.MailtoSettings, body_template="Hello", action_type=type)

        url = reverse("crm_mailto_action", args=[action.id + 1])

        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_mailto_action_next_status(self):
        """send email to the ony contact"""

        type = mommy.make(models.ActionType)
        status2 = mommy.make(models.ActionStatus)
        status1 = mommy.make(models.ActionStatus, next_on_send=status2)
        action = mommy.make(models.Action, subject="Test", type=type, status=status1)
        contact = mommy.make(models.Contact, email="toto@toto.fr")
        action.contacts.add(contact)
        action.save()

        mommy.make(models.MailtoSettings, body_template="Hello", action_type=type)

        url = reverse("crm_mailto_action", args=[action.id])

        response = self.client.get(url)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], "mailto:toto@toto.fr?subject=Test&body=Hello")

        action = models.Action.objects.get(id=action.id)
        self.assertEqual(action.status, status2)



# -*- coding: utf-8 -*-
"""test we can search contacts by action"""

from __future__ import unicode_literals

from django.core.urlresolvers import reverse

from coop_cms.tests import BeautifulSoup
from model_mommy import mommy

from balafon.Crm import models
from balafon.Search.tests import BaseTestCase


class SubscribeAdminContactsTest(BaseTestCase):
    """create actions from results"""

    def setUp(self):
        """before"""
        super(SubscribeAdminContactsTest, self).setUp()
        self.user.is_superuser = True
        self.user.save()

    def test_get_form(self):
        """test GET on url"""
        url = reverse('search_subscribe_contacts_admin')
        response = self.client.get(url)
        self.assertEqual(404, response.status_code)

    def test_view_form(self):
        """test view action form"""

        mommy.make(models.SubscriptionType)
        mommy.make(models.SubscriptionType)

        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = entity1.default_contact
        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = entity2.default_contact

        group = mommy.make(models.Group, name="my group")
        group.entities.add(entity1, entity2)
        group.save()

        url = reverse('search_subscribe_contacts_admin')

        data = {"gr0-_-group-_-0": group.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        id_contacts = soup.select('input#id_contacts')
        self.assertEqual(1, len(id_contacts))
        self.assertEqual(
            sorted([int(x) for x in id_contacts[0]['value'].split(';')]),
            sorted([contact1.id, contact2.id])
        )

        subscription_types = soup.select('select#id_subscription_type option')
        self.assertEqual(2, len(subscription_types))

    def test_view_form_not_logged(self):
        """test view form is not logged"""
        self.client.logout()

        mommy.make(models.SubscriptionType)
        mommy.make(models.SubscriptionType)

        entity1 = mommy.make(models.Entity, name="My tiny corp")
        entity2 = mommy.make(models.Entity, name="Other corp")

        group = mommy.make(models.Group, name="my group")
        group.entities.add(entity1, entity2)
        group.save()

        url = reverse('search_subscribe_contacts_admin')

        data = {"gr0-_-group-_-0": group.id}

        response = self.client.post(url, data=data)
        self.assertEqual(302, response.status_code)
        # login url without lang prefix
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

    def test_subscribe_contacts(self):
        """test create actions for contact"""

        subscription_type1 = mommy.make(models.SubscriptionType)
        subscription_type2 = mommy.make(models.SubscriptionType)

        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = entity1.default_contact
        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = entity2.default_contact
        entity3 = mommy.make(models.Entity, name="Big corp")
        contact3 = entity3.default_contact
        contact4 = mommy.make(models.Contact, entity=entity1)
        contact5 = mommy.make(models.Contact, entity=entity1)

        mommy.make(
            models.Subscription, subscription_type=subscription_type1, contact=contact1, accept_subscription=False
        )
        mommy.make(
            models.Subscription, subscription_type=subscription_type2, contact=contact1, accept_subscription=False
        )
        mommy.make(
            models.Subscription, subscription_type=subscription_type1, contact=contact4, accept_subscription=True
        )
        mommy.make(
            models.Subscription, subscription_type=subscription_type1, contact=contact5, accept_subscription=False
        )

        group = mommy.make(models.Group, name="my group")
        group.entities.add(entity1, entity2)
        group.save()

        contacts = (contact1.id, contact2.id, contact4.id, contact5.id, )

        data = {
            'contacts': ";".join(['{0}'.format(i) for i in contacts]),
            'subscription_type': subscription_type1.id,
            'subscribe': True,
            'subscribe_contacts_admin': True,
        }

        url = reverse('search_subscribe_contacts_admin')
        response = self.client.post(url, data)

        self.assertEqual(200, response.status_code)

        for contact in contacts:
            subscription = models.Subscription.objects.get(subscription_type=subscription_type1, contact=contact)
            self.assertEqual(subscription.accept_subscription, True)

        subscription = models.Subscription.objects.get(subscription_type=subscription_type2, contact=contact1)
        self.assertEqual(subscription.accept_subscription, False)

        try:
            models.Subscription.objects.get(subscription_type=subscription_type1, contact=contact3)
            self.assertTrue(False)  # Error : this subscription should not exist
        except models.Subscription.DoesNotExist:
            pass  # Ok

        try:
            models.Subscription.objects.get(subscription_type=subscription_type2, contact=contact4)
            self.assertTrue(False)  # Error : this subscription should not exist
        except models.Subscription.DoesNotExist:
            pass  # Ok

    def test_unsubscribe_contacts(self):
        """test create actions for contact"""

        subscription_type1 = mommy.make(models.SubscriptionType)
        subscription_type2 = mommy.make(models.SubscriptionType)

        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = entity1.default_contact
        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = entity2.default_contact
        entity3 = mommy.make(models.Entity, name="Big corp")
        contact3 = entity3.default_contact
        contact4 = mommy.make(models.Contact, entity=entity1)
        contact5 = mommy.make(models.Contact, entity=entity1)

        mommy.make(
            models.Subscription, subscription_type=subscription_type1, contact=contact1, accept_subscription=True
        )
        mommy.make(
            models.Subscription, subscription_type=subscription_type2, contact=contact1, accept_subscription=True
        )
        mommy.make(
            models.Subscription, subscription_type=subscription_type1, contact=contact4, accept_subscription=False
        )
        mommy.make(
            models.Subscription, subscription_type=subscription_type1, contact=contact5, accept_subscription=True
        )

        group = mommy.make(models.Group, name="my group")
        group.entities.add(entity1, entity2)
        group.save()

        contacts = (contact1.id, contact2.id, contact4.id, contact5.id,)

        data = {
            'contacts': ";".join(['{0}'.format(i) for i in contacts]),
            'subscription_type': subscription_type1.id,
            'subscribe': False,
            'subscribe_contacts_admin': True,
        }

        url = reverse('search_subscribe_contacts_admin')
        response = self.client.post(url, data)

        self.assertEqual(200, response.status_code)

        for contact in (contact1, contact4, contact5):
            subscription = models.Subscription.objects.get(subscription_type=subscription_type1, contact=contact)
            self.assertEqual(subscription.accept_subscription, False)

        subscription = models.Subscription.objects.get(subscription_type=subscription_type2, contact=contact1)
        self.assertEqual(subscription.accept_subscription, True)

        try:
            models.Subscription.objects.get(subscription_type=subscription_type1, contact=contact3)
            self.assertTrue(False)  # Error : this subscription should not exist
        except models.Subscription.DoesNotExist:
            pass  # Ok

        try:
            models.Subscription.objects.get(subscription_type=subscription_type2, contact=contact4)
            self.assertTrue(False)  # Error : this subscription should not exist
        except models.Subscription.DoesNotExist:
            pass  # Ok

    def test_subscribe_contacts_not_allowed(self):
        """test create actions for contact"""

        self.user.is_superuser = False
        self.user.save()

        subscription_type1 = mommy.make(models.SubscriptionType)
        subscription_type2 = mommy.make(models.SubscriptionType)

        entity1 = mommy.make(models.Entity, name="My tiny corp")
        contact1 = entity1.default_contact
        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = entity2.default_contact
        entity3 = mommy.make(models.Entity, name="Big corp")
        contact3 = entity3.default_contact
        contact4 = mommy.make(models.Contact, entity=entity1)
        contact5 = mommy.make(models.Contact, entity=entity1)

        mommy.make(
            models.Subscription, subscription_type=subscription_type1, contact=contact1, accept_subscription=False
        )
        mommy.make(
            models.Subscription, subscription_type=subscription_type2, contact=contact1, accept_subscription=False
        )
        mommy.make(
            models.Subscription, subscription_type=subscription_type1, contact=contact4, accept_subscription=True
        )
        mommy.make(
            models.Subscription, subscription_type=subscription_type1, contact=contact5, accept_subscription=False
        )

        group = mommy.make(models.Group, name="my group")
        group.entities.add(entity1, entity2)
        group.save()

        contacts = (contact1.id, contact2.id, contact4.id, contact5.id,)

        data = {
            'contacts': ";".join(['{0}'.format(i) for i in contacts]),
            'subscription_type': subscription_type1.id,
            'subscribe': True,
            'subscribe_contacts_admin': True,
        }

        url = reverse('search_subscribe_contacts_admin')
        response = self.client.post(url, data)

        self.assertEqual(302, response.status_code)
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) >= 0)

        subscription = models.Subscription.objects.get(subscription_type=subscription_type1, contact=contact1)
        self.assertEqual(subscription.accept_subscription, False)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from datetime import date


def get_date_bounds(text):
    date1, date2 = text.split(' ')
    date1, date2 = [int(elt) for elt in date1.split('/')], [int(elt) for elt in date2.split('/')]
    date1.reverse(), date2.reverse()
    return date(*date1), date(*date2)



# -*- coding: utf-8 -*-
"""unit testing"""

from __future__ import unicode_literals

from django.core.exceptions import ValidationError

from balafon.utils import validate_rgb
from balafon.Crm.tests import BaseTestCase


class CheckRgbTest(BaseTestCase):
    """check validate_rgb utility"""

    def test_validate_rgb_6_chars(self):
        """it should not raise an error"""
        validate_rgb("#123456")

    def test_validate_rgb_6_hexa(self):
        """it should not raise an error"""
        validate_rgb("#abcdef")

    def test_validate_rgb_3_chars(self):
        """it should not raise an error"""
        validate_rgb("#123")

    def test_validate_rgb_3_hexa(self):
        """it should not raise an error"""
        validate_rgb("#fff")

    def test_validate_rgb_mix(self):
        """it should not raise an error"""
        validate_rgb("#f1b2c3")

    def test_validate_rgb_invalid_char(self):
        """it should raise an error"""
        self.assertRaises(ValidationError, validate_rgb, '#hello')
        self.assertRaises(ValidationError, validate_rgb, 'cool')

    def test_validate_rgb_missing_chars(self):
        """it should raise an error"""
        self.assertRaises(ValidationError, validate_rgb, '#1')
        self.assertRaises(ValidationError, validate_rgb, '#12')
        self.assertRaises(ValidationError, validate_rgb, '#1234')
        self.assertRaises(ValidationError, validate_rgb, '#12345')

    def test_validate_rgb_missing_sharp(self):
        """it should raise an error"""
        self.assertRaises(ValidationError, validate_rgb, '123')
        self.assertRaises(ValidationError, validate_rgb, '123456')

    def test_validate_rgb_too_many_chars(self):
        """it should raise an error"""
        self.assertRaises(ValidationError, validate_rgb, '#1234567')
        self.assertRaises(ValidationError, validate_rgb, '#12345678')



# -*- coding: utf-8 -*-
"""admin site"""

from __future__ import unicode_literals

from django.contrib import admin

import floppyforms.__future__ as forms

from balafon.Crm import settings as crm_settings
from balafon.Emailing import models


@admin.register(models.Emailing)
class EmailingAdmin(admin.ModelAdmin):
    """Emailing"""
    list_display = ['newsletter']
    raw_id_fields = [
        'send_to', 'sent_to', 'opened_emails', 'hard_bounce', 'soft_bounce', 'spam', 'unsub', 'rejected',
    ]

    def get_form(self, *args, **kwargs):
        form_class = super(EmailingAdmin, self).get_form(*args, **kwargs)
        class custom_form_class(form_class):
            def __init__(self, *args, **kwargs):
                super(custom_form_class, self).__init__(*args, **kwargs)
                self.fields['lang'].widget = forms.Select(choices=crm_settings.get_language_choices())
        return custom_form_class


@admin.register(models.MagicLink)
class MagicLinkAdmin(admin.ModelAdmin):
    """Magic link"""
    list_display = ['url', 'emailing']
    search_fields = ['url', 'emailing']
    raw_id_admin = ('emailing',)



# -*- coding: utf-8 -*-
"""a simple store"""

from __future__ import unicode_literals

from django.conf import settings
from django.contrib.sites.models import Site
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.http import Http404, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils.text import slugify
from django.views.generic import TemplateView


from balafon.permissions import can_access
from balafon.settings import get_pdf_view_base_class
from balafon.utils import Utf8JSONRenderer
from balafon.Crm.models import Action
from balafon.Crm.serializers import ActionSerializer
from balafon.Store.models import Discount, Sale, SaleItem, VatRate, StoreManagementActionType
from balafon.Store.api.serializers import (
    DiscountSerializer, SaleItemSerializer, SaleSerializer, VatRateSerializer, VatTotalSerializer
)


class SalesDocumentViewMixin(object):
    """display sales document: to be inherited in TemplateView and PdfTemplateView"""
    action = None
    is_public = False
    is_pdf = False
    template_name = "Store/commercial_document.html"

    def dispatch(self, *args, **kwargs):
        if not self.is_public:
            if not self.request.user.is_authenticated():
                return HttpResponseRedirect(reverse('login'))
        return super(SalesDocumentViewMixin, self).dispatch(*args, **kwargs)

    def get_template_names(self):
        """return customized template of exists"""
        action = self.get_action()
        try:
            store_action_type = StoreManagementActionType.objects.get(action_type=action.type)
            if store_action_type.template_name:
                return store_action_type.template_name
        except StoreManagementActionType.DoesNotExist:
            pass
        return super(SalesDocumentViewMixin, self).get_template_names()

    def get_context_data(self, **kwargs):
        """get context data"""
        context = super(SalesDocumentViewMixin, self).get_context_data(**kwargs)

        if not self.is_public and not can_access(self.request.user):
            raise PermissionDenied

        action = self.get_action()

        sale = self.get_sale(action)

        if self.is_public:
            is_read_only = True
        else:
            if action.frozen:
                is_read_only = True
            else:
                if action.type:
                    is_read_only = action.status in action.type.storemanagementactiontype.readonly_status.all()
                else:
                    is_read_only = False

        action_serializer = ActionSerializer(self.get_action())
        action_data = Utf8JSONRenderer().render(action_serializer.data)

        sale_serializer = SaleSerializer(sale)
        sale_data = Utf8JSONRenderer().render(sale_serializer.data)

        vat_total_serializer = VatTotalSerializer(sale.vat_total_amounts(), many=True)
        vat_total_data = Utf8JSONRenderer().render(vat_total_serializer.data)

        sale_items_serializer = SaleItemSerializer(self.get_sale_items(), many=True)
        sale_items_data = Utf8JSONRenderer().render(sale_items_serializer.data)

        vat_rates_serializer = VatRateSerializer(VatRate.objects.all(), many=True)
        vat_rates_data = Utf8JSONRenderer().render(vat_rates_serializer.data)

        discounts_serializer = DiscountSerializer(Discount.objects.filter(active=True), many=True)
        discounts_data = Utf8JSONRenderer().render(discounts_serializer.data)

        context['action'] = action
        context['is_read_only'] = is_read_only
        context['is_pdf'] = self.is_pdf
        context["references_text"] = sale.get_references_text()

        context['DJANGO_APP'] = """
        var DJANGO_APP = {{
            action: {0},
            sale_items: {1},
            sale: {2},
            vat_rates: {3},
            isPdf: {4},
            isPublic: {5},
            isReadOnly: {6},
            discounts: {7},
            vat_totals: {8},
            showPercentage: {9},
        }};""".format(
            action_data,
            sale_items_data,
            sale_data,
            vat_rates_data,
            'true' if self.is_pdf else 'false',
            'true' if self.is_public else 'false',
            'true' if is_read_only else 'false',
            discounts_data,
            vat_total_data,
            'true' if getattr(settings, 'BALAFON_STORE_SHOW_PERCENTAGE', False) else 'false'
        )
        return context

    def get_action(self):
        """get the associated action"""
        if self.action is None:
            if self.is_public:
                action_uuid = self.kwargs.get('action_uuid', '')
                if not action_uuid:
                    raise Http404
                self.action = get_object_or_404(Action, uuid=action_uuid)
            else:
                action_id = self.kwargs.get('action_id', 0)
                self.action = get_object_or_404(Action, id=action_id)
        return self.action

    def get_sale(self, action):
        try:
            return action.sale
        except Sale.DoesNotExist:
            raise Http404

    def get_sale_items(self):
        """get the sales items"""
        return SaleItem.objects.filter(sale__action=self.get_action())


class SalesDocumentView(SalesDocumentViewMixin, TemplateView):
    """display sales document as html"""
    is_pdf = False


class SalesDocumentPdfView(SalesDocumentViewMixin, get_pdf_view_base_class()):
    """display sales document as pdf"""
    is_pdf = True
    cmd_options = {
        'javascript-delay': 1000,
        'margin-top': 0,
        'margin-bottom': 0,
        'margin-left': 0,
        'margin-right': 0,
    }

    def get_filename(self):
        action = self.get_action()
        return '{0}.pdf'.format(slugify(action.type.name) if action.type else 'document')

    def get_context_data(self, **kwargs):
        """get context data"""
        context = super(SalesDocumentPdfView, self).get_context_data(**kwargs)

        # Make sure that the action is in readonly status
        action = self.get_action()
        default_readonly_status = action.type.storemanagementactiontype.default_readonly_status
        if default_readonly_status:
            read_only_status = action.type.storemanagementactiontype.readonly_status.all()
            if action.status not in read_only_status:
                action.status = default_readonly_status
                action.save()

        if settings.DEBUG:
            port = self.request.META.get('SERVER_PORT', 8000)
            context["DOMAIN"] = 'http://127.0.0.1:{0}'.format(port)
        else:
            try:
                site = Site.objects.get_current()
                context["DOMAIN"] = 'http://{0}'.format(site.domain)
            except Site.DoesNotExist:
                pass

        return context


class SalesDocumentPublicView(SalesDocumentViewMixin, TemplateView):
    """display sales document as pdf"""
    is_pdf = False
    is_public = True
    cmd_options = {
        'javascript-delay': 1000,
        'margin-top': 0,
        'margin-bottom': 0,
        'margin-left': 0,
        'margin-right': 0,
    }


class SalesDocumentPublicPdfView(SalesDocumentPdfView):
    """display sales document as pdf"""
    is_public = True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


def forwards(apps, schema_editor):
    """Set same_as_priority for all same-as contacts """

    # Your migration code goes here
    # We can't import the SameAs model directly as it may be a newer
    # version than this migration expects. We use the historical version.
    same_as_model = apps.get_model("Crm", "SameAs")

    for same_as in same_as_model.objects.all():

        if same_as.contact_set.count() == 1:

            for contact in same_as.contact_set.all():
                contact.same_as = None
                contact.same_as_priority = 0
                contact.save()

        if same_as.contact_set.count() <= 1:

            same_as.delete()


class Migration(migrations.Migration):

    dependencies = [
        ('Crm', '0009_remove_sameas_main_contact'),
    ]

    operations = [
        migrations.RunPython(forwards),
    ]



# -*- coding: utf-8 -*-
"""test we can search contact by zones"""

from __future__ import unicode_literals

from unittest import skipIf

from django.core.urlresolvers import reverse

from coop_cms.tests import BeautifulSoup
from model_mommy import mommy

from balafon.Crm import models
from balafon.Crm import settings as crm_settings
from balafon.Crm.utils import get_default_country

from balafon.Search.tests import BaseTestCase


class ZoneSearchTest(BaseTestCase):
    """Search by zone city, area, country ..."""

    def test_view_search(self):
        """view search form"""
        city1 = mommy.make(models.City)
        city2 = mommy.make(models.City)

        entity1 = mommy.make(models.Entity, city=city1)
        contact1 = entity1.default_contact
        contact1.lastname = "ABCDEFGH"
        contact1.main_contact = True
        contact1.has_left = False
        contact1.save()

        entity2 = mommy.make(models.Entity)
        contact2 = entity2.default_contact
        contact2.lastname = "EFGHIJKL"
        contact2.main_contact = True
        contact2.has_left = False
        contact2.city = city1
        contact2.save()

        entity3 = mommy.make(models.Entity)
        contact3 = entity3.default_contact
        contact3.lastname = "IJKLMNOP"
        contact3.main_contact = True
        contact3.has_left = False
        contact3.city = city2
        contact3.save()

        entity4 = mommy.make(models.Entity)
        contact4 = entity4.default_contact
        contact4.lastname = "MNOPQRST"
        contact4.main_contact = True
        contact4.has_left = False
        contact4.save()

        url = reverse("search_cities", args=[city1.id])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertContains(response, contact1.lastname)
        self.assertContains(response, contact2.lastname)
        self.assertNotContains(response, contact3.lastname)
        self.assertNotContains(response, contact4.lastname)

    def test_search_zipcode(self):
        """search zipcode"""
        city1 = mommy.make(models.City, name="ZooPark")
        city2 = mommy.make(models.City, name="VodooPark")

        entity1 = mommy.make(models.Entity)
        contact1 = entity1.default_contact
        contact1.lastname = "ABCDEFGH"
        contact1.main_contact = True
        contact1.has_left = False
        contact1.city = city1
        contact1.zip_code = "42810"
        contact1.save()

        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKLMNOP", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity)
        contact2 = entity2.default_contact
        contact2.lastname = "DEFGHIJKL"
        contact2.main_contact = True
        contact2.has_left = False
        contact2.city = city2
        contact2.zip_code = "26100"
        contact2.save()

        url = reverse('search')

        data = {"gr0-_-zip_code-_-0": "42"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        self.assertEqual(0, len(soup.select('.field-error')))

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertNotContains(response, contact3.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def test_search_zipcode_entity(self):
        """search zipcode of entity"""
        city1 = mommy.make(models.City, name="ZooPark")
        city2 = mommy.make(models.City, name="VodooPark")

        entity1 = mommy.make(models.Entity, city=city1, zip_code="42810")
        contact1 = entity1.default_contact
        contact1.lastname = "ABCDEFGH"
        contact1.main_contact = True
        contact1.has_left = False
        contact1.save()

        contact3 = mommy.make(
            models.Contact, entity=entity1, lastname="IJKLMNOP",
            main_contact=True, has_left=False
        )

        entity2 = mommy.make(models.Entity, city=city2, zip_code="26100")
        contact2 = entity2.default_contact
        contact2.lastname = "DEFGHIJKL"
        contact2.main_contact = True
        contact2.has_left = False
        contact2.save()

        url = reverse('search')

        data = {"gr0-_-zip_code-_-0": "42"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        self.assertEqual(0, len(soup.select('.field-error')))

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def test_search_zipcode_entity_contact_mix(self):
        """zipcode entity and contact"""
        city1 = mommy.make(models.City, name="ZooPark")
        city2 = mommy.make(models.City, name="VodooPark")

        entity1 = mommy.make(models.Entity, city=city1, zip_code="42810")
        contact1 = entity1.default_contact
        contact1.lastname = "ABCDEFGH"
        contact1.main_contact = True
        contact1.has_left = False
        contact1.save()

        contact3 = mommy.make(
            models.Contact, city=None, entity=entity1, lastname="IJKLMNOP",
            main_contact=True, has_left=False
        )
        contact4 = mommy.make(
            models.Contact, city=city2, zip_code="26100", entity=entity1,
            lastname="MNOPQRST", main_contact=True, has_left=False
        )

        entity2 = mommy.make(models.Entity)
        contact2 = entity2.default_contact
        contact2.lastname = "DEFGHIJKL"
        contact2.main_contact = True
        contact2.has_left = False
        contact2.city = city2
        contact2.zip_code = "26100"
        contact2.save()

        url = reverse('search')

        data = {"gr0-_-zip_code-_-0": "42"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        self.assertEqual(0, len(soup.select('.field-error')))

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)
        self.assertNotContains(response, contact4.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def test_search_entity_zipcode(self):
        """another zipcode search"""
        city1 = mommy.make(models.City, name="ZooPark")
        city2 = mommy.make(models.City, name="VodooPark")

        entity1 = mommy.make(models.Entity, city=city1, zip_code="42810")
        contact1 = entity1.default_contact
        contact1.lastname = "ABCDEFGH"
        contact1.main_contact = True
        contact1.has_left = False
        contact1.save()

        contact3 = mommy.make(
            models.Contact, city=None, entity=entity1, lastname="IJKLMNOP",
            main_contact=True, has_left=False
        )
        contact4 = mommy.make(
            models.Contact, city=city2, zip_code="26100", entity=entity1,
            lastname="MNOPQRST", main_contact=True, has_left=False
        )

        entity2 = mommy.make(models.Entity, city=city2, zip_code="26100")
        contact2 = entity2.default_contact
        contact2.lastname = "DEFGHIJKL"
        contact2.main_contact = True
        contact2.has_left = False
        contact2.save()

        url = reverse('search')

        data = {"gr0-_-entity_zip_code-_-0": "42"}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        self.assertEqual(0, len(soup.select('.field-error')))

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)
        self.assertContains(response, contact4.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def test_search_city(self, cities=None, data=None):
        """search by city"""
        if cities:
            city1 = cities[0]
            city2 = cities[1]
        else:
            city1 = mommy.make(models.City, name="ZooPark")
            city2 = mommy.make(models.City, name="VodooPark")

        entity1 = mommy.make(models.Entity)
        contact1 = entity1.default_contact
        contact1.lastname = "ABCDEFGH"
        contact1.main_contact = True
        contact1.has_left = False
        contact1.city = city1
        contact1.save()

        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKLMNOP", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity)
        contact2 = entity2.default_contact
        contact2.lastname = "DEFGHIJK"
        contact2.main_contact = True
        contact2.has_left = False
        contact2.city = city2
        contact2.save()

        url = reverse('search')

        data = data or {"gr0-_-city-_-0": city1.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        self.assertEqual(0, len(soup.select('.field-error')))

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertNotContains(response, contact3.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def test_search_city_entity(self, cities=None, data=None):
        """search by city of the entity"""
        if cities:
            city1 = cities[0]
            city2 = cities[1]
        else:
            city1 = mommy.make(models.City, name="ZooPark")
            city2 = mommy.make(models.City, name="VodooPark")

        entity1 = mommy.make(models.Entity, city=city1)
        contact1 = entity1.default_contact
        contact1.lastname = "ABCDEFGH"
        contact1.main_contact = True
        contact1.has_left = False
        contact1.save()

        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKLMNOP", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, city=city2)
        contact2 = entity2.default_contact
        contact2.lastname = "DEFGHIJKL"
        contact2.main_contact = True
        contact2.has_left = False
        contact2.save()

        url = reverse('search')

        data = data or {"gr0-_-city-_-0": city1.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        self.assertEqual(0, len(soup.select('.field-error')))

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def test_search_city_entity_contact_mix(self, cities=None, data=None):
        """search by city for entity and contact"""
        if cities:
            city1 = cities[0]
            city2 = cities[1]
        else:
            city1 = mommy.make(models.City, name="ZooPark")
            city2 = mommy.make(models.City, name="VodooPark")

        entity1 = mommy.make(models.Entity, city=city1)
        contact1 = entity1.default_contact
        contact1.lastname = "ABCDEFGH"
        contact1.main_contact = True
        contact1.has_left = False
        contact1.save()

        contact3 = mommy.make(
            models.Contact, city=None, entity=entity1, lastname="IJKLMNOP", main_contact=True, has_left=False
        )
        contact4 = mommy.make(
            models.Contact, city=city2, entity=entity1, lastname="MNOPQRST", main_contact=True, has_left=False
        )

        entity2 = mommy.make(models.Entity)
        contact2 = entity2.default_contact
        contact2.lastname = "DEFGHIJKL"
        contact2.main_contact = True
        contact2.has_left = False
        contact2.city = city2
        contact2.save()

        url = reverse('search')

        data = data or {"gr0-_-city-_-0": city1.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        self.assertEqual(0, len(soup.select('.field-error')))

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)
        self.assertNotContains(response, contact4.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def _test_search_multi_zones(self, cities=None, data=None, entity_search=False):
        """search by zone"""
        city1 = cities[0]
        city2 = cities[1]
        city3 = mommy.make(models.City, name="BlablaPark")

        entity1 = mommy.make(models.Entity, city=city1 if entity_search else None)
        contact1a = entity1.default_contact
        contact1a.lastname = "ABCDEFGH"
        contact1a.main_contact = True
        contact1a.has_left = False
        contact1a.city = city1
        contact1a.save()

        contact1b = mommy.make(models.Contact, entity=entity1, lastname="IJKLMNOP", main_contact=True, has_left=False)

        entity2 = mommy.make(models.Entity, city=city2 if entity_search else None)
        contact2 = entity2.default_contact
        contact2.lastname = "DEFGHIJKL"
        contact2.main_contact = True
        contact2.has_left = False
        contact2.city = None if entity_search else city2
        contact2.save()

        entity3 = mommy.make(models.Entity)
        contact3 = entity3.default_contact
        contact3.lastname = "MNOPQRST"
        contact3.main_contact = True
        contact3.has_left = False
        contact3.city = city3
        contact3.save()

        url = reverse('search')

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        self.assertEqual([], soup.select('.field-error'))

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1a.lastname)
        if entity_search:
            self.assertContains(response, contact1b.lastname)
        else:
            self.assertNotContains(response, contact1b.lastname)

        self.assertContains(response, entity2.name)
        self.assertContains(response, contact2.lastname)

        self.assertNotContains(response, entity3.name)
        self.assertNotContains(response, contact3.lastname)

    def test_search_entity_city(self, cities=None, data=None):
        """another city serach for entity"""
        if cities:
            city1 = cities[0]
            city2 = cities[1]
        else:
            city1 = mommy.make(models.City, name="ZooPark")
            city2 = mommy.make(models.City, name="VodooPark")

        entity1 = mommy.make(models.Entity, city=city1)
        contact1 = entity1.default_contact
        contact1.lastname = "ABCDEFGH"
        contact1.main_contact = True
        contact1.has_left = False
        contact1.save()

        contact3 = mommy.make(
            models.Contact, city=None, entity=entity1, lastname="IJKLMNOP", main_contact=True, has_left=False
        )
        contact4 = mommy.make(
            models.Contact, city=city2, entity=entity1, lastname="MNOPQRST", main_contact=True, has_left=False
        )

        entity2 = mommy.make(models.Entity, city=city2)
        contact2 = entity2.default_contact
        contact2.lastname = "DEFGHIJKL"
        contact2.main_contact = True
        contact2.has_left = False
        contact2.save()

        url = reverse('search')

        data = data or {"gr0-_-entity_city-_-0": city1.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        self.assertEqual(0, len(soup.select('.field-error')))

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)
        self.assertContains(response, contact4.lastname)

        self.assertNotContains(response, entity2.name)
        self.assertNotContains(response, contact2.lastname)

    def _get_departements_data(self, form_name="department"):
        """get departments"""
        default_country = get_default_country()
        region_type = mommy.make(models.ZoneType, type="region")
        region = mommy.make(models.Zone, parent=default_country, type=region_type)
        departement_type = mommy.make(models.ZoneType, type="department")
        departement1 = mommy.make(models.Zone, parent=region, type=departement_type)
        departement2 = mommy.make(models.Zone, parent=region, type=departement_type)

        city1 = mommy.make(models.City, name="ZooPark", parent=departement1)
        city2 = mommy.make(models.City, name="VodooPark", parent=departement2)

        data = {"gr0-_-{0}-_-0".format(form_name): departement1.id}

        return (city1, city2), data

    def test_search_multi_departements(self):
        """search by several departements"""
        cities, data = self._get_departements_data()
        city1 = cities[0]
        city2 = cities[1]
        data = {"gr0-_-department-_-0": [city1.parent.id, city2.parent.id]}
        self._test_search_multi_zones(cities, data)

    def test_search_multi_departements_entity(self):
        """search by several entity departements"""
        cities, data = self._get_departements_data()
        city1 = cities[0]
        city2 = cities[1]
        data = {"gr0-_-entity_department-_-0": [city1.parent.id, city2.parent.id]}
        self._test_search_multi_zones(cities, data, True)

    def test_search_departement(self):
        """search by departement"""
        self.test_search_city(*self._get_departements_data())

    def test_search_departement_entity(self):
        """search by entity departements"""
        self.test_search_city_entity(*self._get_departements_data())

    def test_search_departement_entity_contact(self):
        """search by departements of contacts and entities"""
        self.test_search_city_entity_contact_mix(*self._get_departements_data())

    def test_search_entity_departement(self):
        """search by entity departements"""
        self.test_search_entity_city(*self._get_departements_data("entity_department"))

    def _get_regions_data(self, form_name="region"):
        """get region data"""
        default_country = get_default_country()
        region_type = mommy.make(models.ZoneType, type="region")
        region1 = mommy.make(models.Zone, parent=default_country, type=region_type)
        region2 = mommy.make(models.Zone, parent=default_country, type=region_type)
        departement_type = mommy.make(models.ZoneType, type="department")
        departement1 = mommy.make(models.Zone, parent=region1, type=departement_type)
        departement2 = mommy.make(models.Zone, parent=region2, type=departement_type)

        city1 = mommy.make(models.City, name="ZooPark", parent=departement1)
        city2 = mommy.make(models.City, name="VodooPark", parent=departement2)

        data = {"gr0-_-{0}-_-0".format(form_name): region1.id}

        return (city1, city2), data

    def test_search_region(self):
        """search by region"""
        self.test_search_city(*self._get_regions_data())

    def test_search_multi_regions(self):
        """search by several regions"""
        cities, data = self._get_regions_data()
        city1 = cities[0]
        city2 = cities[1]
        data = {"gr0-_-region-_-0": [city1.parent.parent.id, city2.parent.parent.id]}
        self._test_search_multi_zones(cities, data)

    def test_search_region_entity(self):
        """search by region entity"""
        self.test_search_city_entity(*self._get_regions_data())

    def test_search_region_entity_contact_mix(self):
        """search by region contacts and entities"""
        self.test_search_city_entity_contact_mix(*self._get_regions_data())

    def test_search_entity_region(self):
        """search by entity region"""
        self.test_search_entity_city(*self._get_regions_data("entity_region"))

    def _get_countries_data(self, form_name="country"):
        """get countries data"""
        country_type = mommy.make(models.ZoneType, type="country")
        country1 = mommy.make(models.Zone, type=country_type)
        country2 = mommy.make(models.Zone, type=country_type)

        city1 = mommy.make(models.City, name="ZooPark", parent=country1)
        city2 = mommy.make(models.City, name="VodooPark", parent=country2)

        data = {"gr0-_-{0}-_-0".format(form_name): country1.id}

        return (city1, city2), data

    def test_search_country(self):
        """search by country"""
        self.test_search_city(*self._get_countries_data())

    def test_search_country_entity(self):
        """search by country entity"""
        self.test_search_city_entity(*self._get_countries_data())

    def test_search_multi_country(self):
        """search by several countries"""
        cities, data = self._get_countries_data()
        city1 = cities[0]
        city2 = cities[1]
        data = {"gr0-_-country-_-0": [city1.parent.id, city2.parent.id]}
        self._test_search_multi_zones(cities, data)

    def test_search_country_entity_contact_mix(self):
        """search by contact and entity country"""
        self.test_search_city_entity_contact_mix(*self._get_countries_data())

    def test_search_entity_country(self):
        """search by entity country"""
        self.test_search_entity_city(*self._get_countries_data("entity_country"))

    def _get_countries_mix_data(self, form_name="country"):
        """get date with full hierarchy"""
        country_type = mommy.make(models.ZoneType, type="country")
        country1 = default_country = get_default_country()
        country2 = mommy.make(models.Zone, type=country_type, parent=None)
        region_type = mommy.make(models.ZoneType, type="region")
        region1 = mommy.make(models.Zone, parent=default_country, type=region_type)
        departement_type = mommy.make(models.ZoneType, type="department")
        departement1 = mommy.make(models.Zone, parent=region1, type=departement_type)

        city1 = mommy.make(models.City, name="ZooPark", parent=departement1)
        city2 = mommy.make(models.City, name="VodooPark", parent=country2)

        data = {"gr0-_-{0}-_-0".format(form_name): country1.id}

        return (city1, city2), data

    def test_search_country2(self):
        """search by country full hierarchy"""
        self.test_search_city(*self._get_countries_mix_data())

    def test_search_country_entity2(self):
        """search by entity country full hierarchy"""
        self.test_search_city_entity(*self._get_countries_mix_data())

    def test_search_country_entity_contact_mi2x(self):
        """search by entity and contact country full hierarchy"""
        self.test_search_city_entity_contact_mix(*self._get_countries_mix_data())

    def test_search_entity_country2(self):
        """again search by entity country full hierarchy"""
        self.test_search_entity_city(*self._get_countries_mix_data("entity_country"))

    def _get_zonegroup_data(self, form_name="zone_group"):
        """get zonegroup data"""
        default_country = get_default_country()

        region_type = mommy.make(models.ZoneType, type="region")
        region1 = mommy.make(models.Zone, parent=default_country, type=region_type)
        region2 = mommy.make(models.Zone, parent=default_country, type=region_type)
        departement_type = mommy.make(models.ZoneType, type="department")
        departement1 = mommy.make(models.Zone, parent=region1, type=departement_type)
        departement2 = mommy.make(models.Zone, parent=region2, type=departement_type)

        city1 = mommy.make(models.City, name="ZooPark", parent=departement1)
        city2 = mommy.make(models.City, name="VodooPark", parent=departement2)

        zone_group_type = mommy.make(models.ZoneType, type="zone_group")
        zone_group = mommy.make(models.Zone, parent=None, type=zone_group_type)

        city1.groups.add(zone_group)
        city1.save()

        data = {"gr0-_-{0}-_-0".format(form_name): zone_group.id}

        return (city1, city2), data

    @skipIf(not crm_settings.ZONE_GROUP_SEARCH, "ZONE_GROUP_SEARCH disabled")
    def test_search_zonegroup(self):
        """search by contact zone group"""
        self.test_search_city(*self._get_zonegroup_data())

    @skipIf(not crm_settings.ZONE_GROUP_SEARCH, "ZONE_GROUP_SEARCH disabled")
    def test_search_zonegroup_entity(self):
        """search by entity zone group"""
        self.test_search_city_entity(*self._get_zonegroup_data())

    @skipIf(not crm_settings.ZONE_GROUP_SEARCH, "ZONE_GROUP_SEARCH disabled")
    def test_search_zonegroup_entity_contact_mix(self):
        """search by contact and entity zone group"""
        self.test_search_city_entity_contact_mix(*self._get_zonegroup_data())


class CitySearchTest(BaseTestCase):
    """Another city search test"""

    def test_search_city(self):
        """search by city"""
        city = mommy.make(models.City)
        entity1 = mommy.make(models.Entity, name="My tiny corp", city=city)
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCDEFGH", email="toto1@toto.fr")
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKLMNOP")

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(models.Contact, entity=entity2, lastname="WXYZ", city=city)

        entity3 = mommy.make(models.Entity, name="The big Org", email="toto2@toto.fr")
        contact4 = mommy.make(models.Contact, entity=entity3, lastname="ABCABC")

        url = reverse('search')

        data = {"gr0-_-city-_-0": city.id}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)

        self.assertContains(response, entity2.name)
        self.assertContains(response, contact2.lastname)

        self.assertNotContains(response, entity3.name)
        self.assertNotContains(response, contact4.lastname)


class AddressSearchTest(BaseTestCase):
    """search by address"""

    def test_search_address(self):
        """search by city"""
        city = mommy.make(models.City)
        entity1 = mommy.make(models.Entity, name="My tiny corp", city=city, address="rue Paul Mc Cartney")
        contact1 = mommy.make(models.Contact, entity=entity1, lastname="ABCDEFGH", email="toto1@toto.fr")
        contact3 = mommy.make(models.Contact, entity=entity1, lastname="IJKLMNOP")

        entity2 = mommy.make(models.Entity, name="Other corp")
        contact2 = mommy.make(
            models.Contact, entity=entity2, lastname="WXYZ", city=city, address="rue Jean-Paul Belmondo"
        )

        entity3 = mommy.make(models.Entity, name="The big Org", email="toto2@toto.fr")
        contact4 = mommy.make(models.Contact, entity=entity3, lastname="ABCABC")

        entity5 = mommy.make(models.Entity, is_single_contact=True)
        contact5 = entity5.default_contact
        contact5.lastname = "QWERTYUIOP"
        contact5.address = "lot appaulou"
        contact5.city = city
        contact5.save()

        url = reverse('search')

        data = {"gr0-_-address-_-0": 'Paul'}

        response = self.client.post(url, data=data)
        self.assertEqual(200, response.status_code)

        self.assertContains(response, entity1.name)
        self.assertContains(response, contact1.lastname)
        self.assertContains(response, contact3.lastname)

        self.assertContains(response, entity2.name)
        self.assertContains(response, contact2.lastname)

        self.assertNotContains(response, entity3.name)
        self.assertNotContains(response, contact4.lastname)

        self.assertContains(response, contact5.lastname)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib import admin

from balafon.Search import models


@admin.register(models.Search)
class SearchAdmin(admin.ModelAdmin):
    pass


@admin.register(models.SearchGroup)
class SearchGroupAdmin(admin.ModelAdmin):
    list_display = ['name', 'search']
    search_fields = ['name', 'search']
    raw_id_admin = ('search',)


@admin.register(models.SearchField)
class SearchFieldAdmin(admin.ModelAdmin):
    list_display = ['field', 'search_group', 'value']
    search_fields = ['field', 'field__group__search']
    raw_id_admin = ('search_group',)



# -*- coding: utf-8 -*-
"""test subscription"""

from __future__ import unicode_literals

from datetime import date

from django.conf import settings
from django.contrib.sites.models import Site
from django.core import mail
from django.core.urlresolvers import reverse
from django.test.utils import override_settings
from django.utils.translation import activate

from coop_cms.tests import BeautifulSoup
from model_mommy import mommy

from balafon.unit_tests import TestCase
from balafon.Crm import models
from balafon.Crm.signals import new_subscription


@override_settings(BALAFON_EMAIL_SUBSCRIBE_ENABLED=True)
class SubscribeTest(TestCase):
    """Subscription with just his email address"""

    @staticmethod
    def email_as_text(email):
        return email.message().as_string()

    def setUp(self):
        """before each test"""
        super(SubscribeTest, self).setUp()
        self._lang = settings.LANGUAGES[0][0]
        activate(self._lang)

        if not getattr(settings, 'BALAFON_ALLOW_SINGLE_CONTACT', True):
            settings.BALAFON_INDIVIDUAL_ENTITY_ID = models.EntityType.objects.create(name="particulier").id

        self._default_subscription_type = getattr(settings, "BALAFON_DEFAULT_SUBSCRIPTION_TYPE", None)

        mommy.make(models.Zone, name=settings.BALAFON_DEFAULT_COUNTRY, parent=None)

    def tearDown(self):
        """after each test"""
        activate(self._lang)
        settings.BALAFON_DEFAULT_SUBSCRIPTION_TYPE = self._default_subscription_type
        super(SubscribeTest, self).tearDown()

    @override_settings(LANGUAGES=(('en', 'English'), ('fr', 'French')))
    def test_view_subscription_language(self):
        """make sure the favorite_language is set correctly"""
        activate('fr')

        mommy.make(models.SubscriptionType)

        url = reverse("emailing_email_subscribe_newsletter")

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)

        self.assertEqual(1, len(soup.select("#id_favorite_language")))
        self.assertEqual('fr', soup.select("#id_favorite_language")[0]["value"])

    @override_settings(LANGUAGES=(('en', 'English'),))
    def test_view_subscription_no_language(self):
        """make sure the favorite_language is set correctly"""
        mommy.make(models.SubscriptionType)

        url = reverse("emailing_email_subscribe_newsletter")

        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        soup = BeautifulSoup(response.content)

        self.assertEqual(1, len(soup.select("#id_favorite_language")))
        self.assertEqual('', soup.select("#id_favorite_language")[0].get("value", ""))

    def test_view_subscribe_newsletter(self):
        """view email subscription page"""
        url = reverse("emailing_email_subscribe_newsletter")
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 0)
        soup = BeautifulSoup(response.content)
        self.assertEqual(1, len(soup.select('form')))
        form = soup.select('form')[0]
        self.assertEqual(form['action'], url)

    def test_view_subscribe_newsletter_by_id(self):
        """view email subscription page"""
        site1 = Site.objects.get_current()
        subscription_type1 = mommy.make(models.SubscriptionType, name="abc", site=site1)
        url = reverse("emailing_email_subscribe_subscription", args=[subscription_type1.id])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 0)
        soup = BeautifulSoup(response.content)
        self.assertEqual(1, len(soup.select('form')))
        form = soup.select('form')[0]
        self.assertEqual(form['action'], url)
        
    @override_settings(BALAFON_DEFAULT_SUBSCRIPTION_TYPE=None)
    def test_email_subscribe_newsletter(self):
        """test email subscription when no type is configured"""
        url = reverse("emailing_email_subscribe_newsletter")
        
        site1 = Site.objects.get_current()
        site2 = mommy.make(Site)
        
        subscription_type1 = mommy.make(models.SubscriptionType, name="abc", site=site1)
        subscription_type2 = mommy.make(models.SubscriptionType, name="def", site=site1)
        subscription_type3 = mommy.make(models.SubscriptionType, name="ghi", site=site2)
        subscription_type4 = mommy.make(models.SubscriptionType, name="jkl")
        
        data = {
            'email': 'pdupond@apidev.fr',
        }
        
        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 1)
        
        contact = models.Contact.objects.all()[0]
        
        self.assertNotEqual(contact.uuid, '')
        self.assertTrue(response['Location'].find(reverse('emailing_subscribe_email_done')) >= 0)
        
        self.assertEqual(contact.email, data['email'])
        
        self.assertEqual(len(mail.outbox), 2)
        
        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [contact.email])
        url = reverse('emailing_email_verification', args=[contact.uuid])
        email_content = self.email_as_text(verification_email)
        self.assertTrue(email_content.find(url) > 0)
        
        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])
        # Not an Error message
        self.assertTrue(self.email_as_text(notification_email).find("Error") < 0)
        
        for subscription_type in (subscription_type1, subscription_type2):
            subscription = models.Subscription.objects.get(contact=contact, subscription_type=subscription_type)
            self.assertEqual(subscription.accept_subscription, True)
            self.assertEqual(subscription.subscription_date.date(), date.today())

        for subscription_type in (subscription_type3, subscription_type4):
            queryset = models.Subscription.objects.filter(contact=contact, subscription_type=subscription_type)
            self.assertEqual(0, queryset.count())
        
    @override_settings(BALAFON_DEFAULT_SUBSCRIPTION_TYPE=None)
    def test_email_subscribe_no_email(self):
        """test email subscription with empty email should fail"""
        url = reverse("emailing_email_subscribe_newsletter")

        data = {
            'email': '',
        }

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select("ul.errorlist")), 1)
        self.assertEqual(models.Contact.objects.count(), 0)
        self.assertEqual(len(mail.outbox), 0)

    @override_settings(BALAFON_DEFAULT_SUBSCRIPTION_TYPE=None)
    def test_email_subscribe_invalid_email(self):
        """email subscription with an invalid email should fail"""

        url = reverse("emailing_email_subscribe_newsletter")
        
        data = {
            'email': 'coucou',
        }
        
        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(200, response.status_code)
        soup = BeautifulSoup(response.content)
        self.assertEqual(len(soup.select("ul.errorlist")), 1)
        self.assertEqual(models.Contact.objects.count(), 0)
        
        self.assertEqual(len(mail.outbox), 0)

    @override_settings(BALAFON_DEFAULT_SUBSCRIPTION_TYPE=None)
    def test_email_subscribe_default_subscription(self):
        """email subscription should set type to the default subscription type"""

        url = reverse("emailing_email_subscribe_newsletter")

        site1 = Site.objects.get_current()
        site2 = mommy.make(Site)

        subscription_type1 = mommy.make(models.SubscriptionType, name="abc", site=site1)
        subscription_type2 = mommy.make(models.SubscriptionType, name="def", site=site1)
        subscription_type3 = mommy.make(models.SubscriptionType, name="ghi", site=site2)
        subscription_type4 = mommy.make(models.SubscriptionType, name="jkl")

        settings.BALAFON_DEFAULT_SUBSCRIPTION_TYPE = subscription_type1.id

        data = {
            'email': 'pdupond@apidev.fr',
        }

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 1)

        contact = models.Contact.objects.all()[0]

        self.assertNotEqual(contact.uuid, '')
        self.assertTrue(response['Location'].find(reverse('emailing_subscribe_email_done')) >= 0)

        self.assertEqual(contact.email, data['email'])

        self.assertEqual(len(mail.outbox), 2)

        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [contact.email])
        url = reverse('emailing_email_verification', args=[contact.uuid])
        email_content = self.email_as_text(verification_email)
        self.assertTrue(email_content.find(url) > 0)

        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])
        #Not an Error message
        self.assertTrue(self.email_as_text(notification_email).find("Error") < 0)

        for subscription_type in (subscription_type1, ):
            subscription = models.Subscription.objects.get(contact=contact, subscription_type=subscription_type)
            self.assertEqual(subscription.accept_subscription, True)
            self.assertEqual(subscription.subscription_date.date(), date.today())

        for subscription_type in (subscription_type2, subscription_type3, subscription_type4):
            queryset = models.Subscription.objects.filter(contact=contact, subscription_type=subscription_type)
            self.assertEqual(0, queryset.count())

    @override_settings(BALAFON_DEFAULT_SUBSCRIPTION_TYPE=None)
    def test_email_subscribe_selected(self):
        """test subscribe for the given subscription type"""

        site1 = Site.objects.get_current()
        site2 = mommy.make(Site)

        subscription_type1 = mommy.make(models.SubscriptionType, name="abc", site=site1)
        subscription_type2 = mommy.make(models.SubscriptionType, name="def", site=site1)
        subscription_type3 = mommy.make(models.SubscriptionType, name="ghi", site=site2)
        subscription_type4 = mommy.make(models.SubscriptionType, name="jkl")

        url = reverse("emailing_email_subscribe_subscription", args=[subscription_type1.id])

        data = {
            'email': 'pdupond@apidev.fr',
        }

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 1)

        contact = models.Contact.objects.all()[0]

        self.assertNotEqual(contact.uuid, '')
        self.assertTrue(response['Location'].find(reverse('emailing_subscribe_email_done')) >= 0)

        self.assertEqual(contact.email, data['email'])

        self.assertEqual(len(mail.outbox), 2)

        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [contact.email])
        url = reverse('emailing_email_verification', args=[contact.uuid])
        email_content = self.email_as_text(verification_email)
        self.assertTrue(email_content.find(url) > 0)

        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])
        # Not an Error message
        self.assertTrue(self.email_as_text(notification_email).find("Error") < 0)

        for subscription_type in (subscription_type1, ):
            subscription = models.Subscription.objects.get(contact=contact, subscription_type=subscription_type)
            self.assertEqual(subscription.accept_subscription, True)
            self.assertEqual(subscription.subscription_date.date(), date.today())

        for subscription_type in (subscription_type2, subscription_type3, subscription_type4):
            queryset = models.Subscription.objects.filter(contact=contact, subscription_type=subscription_type)
            self.assertEqual(0, queryset.count())

    @override_settings(BALAFON_DEFAULT_SUBSCRIPTION_TYPE=None)
    def test_email_subscribe_selected_and_default(self):
        """test given subscribe type has priority on default one"""

        site1 = Site.objects.get_current()
        site2 = mommy.make(Site)

        subscription_type1 = mommy.make(models.SubscriptionType, name="abc", site=site1)
        subscription_type2 = mommy.make(models.SubscriptionType, name="def", site=site1)
        subscription_type3 = mommy.make(models.SubscriptionType, name="ghi", site=site2)
        subscription_type4 = mommy.make(models.SubscriptionType, name="jkl")

        settings.BALAFON_DEFAULT_SUBSCRIPTION_TYPE = subscription_type1.id
        url = reverse("emailing_email_subscribe_subscription", args=[subscription_type2.id])

        data = {
            'email': 'pdupond@apidev.fr',
        }

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 1)

        contact = models.Contact.objects.all()[0]

        self.assertNotEqual(contact.uuid, '')
        self.assertTrue(response['Location'].find(reverse('emailing_subscribe_email_done')) >= 0)

        self.assertEqual(contact.email, data['email'])

        self.assertEqual(len(mail.outbox), 2)

        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [contact.email])
        url = reverse('emailing_email_verification', args=[contact.uuid])
        email_content = self.email_as_text(verification_email)
        self.assertTrue(email_content.find(url) > 0)

        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])
        #Not an Error message
        self.assertTrue(self.email_as_text(notification_email).find("Error") < 0)

        for subscription_type in (subscription_type2, ):
            subscription = models.Subscription.objects.get(contact=contact, subscription_type=subscription_type)
            self.assertEqual(subscription.accept_subscription, True)
            self.assertEqual(subscription.subscription_date.date(), date.today())

        for subscription_type in (subscription_type1, subscription_type3, subscription_type4):
            queryset = models.Subscription.objects.filter(contact=contact, subscription_type=subscription_type)
            self.assertEqual(0, queryset.count())

    @override_settings(BALAFON_DEFAULT_SUBSCRIPTION_TYPE=None)
    def test_email_subscribe_selected_not_in_site(self):
        """test subscription on a type which is not associated with the current ste"""
        site1 = Site.objects.get_current()
        site2 = mommy.make(Site)

        subscription_type1 = mommy.make(models.SubscriptionType, name="abc", site=site1)
        mommy.make(models.SubscriptionType, name="def", site=site1)
        subscription_type3 = mommy.make(models.SubscriptionType, name="ghi", site=site2)
        mommy.make(models.SubscriptionType, name="jkl")

        settings.BALAFON_DEFAULT_SUBSCRIPTION_TYPE = subscription_type1.id
        url = reverse("emailing_email_subscribe_subscription", args=[subscription_type3.id])

        data = {
            'email': 'pdupond@apidev.fr',
        }

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 1)

        contact = models.Contact.objects.all()[0]

        self.assertNotEqual(contact.uuid, '')
        self.assertTrue(response['Location'].find(reverse('emailing_subscribe_email_done')) >= 0)

        self.assertEqual(contact.email, data['email'])

        self.assertEqual(len(mail.outbox), 2)

        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [contact.email])
        url = reverse('emailing_email_verification', args=[contact.uuid])
        email_content = self.email_as_text(verification_email)
        self.assertTrue(email_content.find(url) > 0)

        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])
        #Error message
        self.assertTrue(self.email_as_text(notification_email).find("Error") >= 0)

        queryset = models.Subscription.objects.filter(contact=contact)
        self.assertEqual(0, queryset.count())

    def test_subscribe_existing_contact(self):
        """subscribe again to mailing list"""
        entity = mommy.make(models.Entity)
        existing_contact = entity.default_contact
        existing_contact.email = 'pdupond@apidev.fr'
        existing_contact.save()

        subscription_type1 = mommy.make(models.SubscriptionType, name="abc", site=Site.objects.get_current())
        url = reverse("emailing_email_subscribe_subscription", args=[subscription_type1.id])

        data = {
            'email': existing_contact.email,
        }

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 2)
        self.assertEqual(models.Contact.objects.filter(email=existing_contact.email).count(), 2)

        new_contact = models.Contact.objects.filter(email=existing_contact.email).exclude(id=existing_contact.id)[0]
        self.assertEqual(new_contact.email_verified, False)

        self.assertEqual(new_contact.get_same_as().count(), 0)
        self.assertEqual(new_contact.get_same_email().count(), 1)

        self.assertNotEqual(new_contact.uuid, '')
        self.assertEqual(new_contact.favorite_language, '')
        self.assertTrue(response['Location'].find(reverse('emailing_subscribe_email_done')) >= 0)

        queryset = models.Subscription.objects.filter(contact=new_contact, subscription_type=subscription_type1)
        self.assertEqual(1, queryset.count())

        #email verification
        self.assertEqual(len(mail.outbox), 2)

        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [new_contact.email])
        url = reverse('emailing_email_verification', args=[new_contact.uuid])
        email_content = self.email_as_text(verification_email)
        self.assertTrue(email_content.find(url) > 0)

        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])
        email_content = self.email_as_text(notification_email)
        self.assertTrue(email_content.find(new_contact.fullname) > 0)
        self.assertTrue(email_content.find(new_contact.get_absolute_url()) > 0)
        self.assertTrue(email_content.find(existing_contact.get_absolute_url()) > 0)

    def test_subscribe_several_existing_contact(self):
        """subscribe again to mailing list"""
        entity1 = mommy.make(models.Entity)
        existing_contact1 = entity1.default_contact
        existing_contact1.email = 'pdupond@apidev.fr'
        existing_contact1.save()

        entity2 = mommy.make(models.Entity, email=existing_contact1.email)
        existing_contact2 = entity2.default_contact
        existing_contact4 = mommy.make(models.Contact, entity=entity2)

        entity3 = mommy.make(models.Entity)
        existing_contact3 = entity3.default_contact
        existing_contact3.email = existing_contact1.email
        existing_contact3.save()

        mommy.make(models.Contact, email="toto@somethingelse.fr")

        subscription_type1 = mommy.make(models.SubscriptionType, name="abc", site=Site.objects.get_current())
        url = reverse("emailing_email_subscribe_subscription", args=[subscription_type1.id])

        data = {
            'email': existing_contact1.email,
        }

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)
        self.assertEqual(models.Contact.objects.filter(email=existing_contact1.email).count(), 3)

        existing_ids = [
            contact.id for contact in (existing_contact1, existing_contact2, existing_contact3, existing_contact4)
        ]
        queryset = models.Contact.objects.filter(email=existing_contact1.email).exclude(id__in=existing_ids)
        self.assertEqual(queryset.count(), 1)
        new_contact = queryset[0]
        self.assertEqual(new_contact.email_verified, False)

        queryset = models.Subscription.objects.filter(contact=new_contact, subscription_type=subscription_type1)
        self.assertEqual(1, queryset.count())

        self.assertEqual(new_contact.get_same_as().count(), 0)
        self.assertEqual(new_contact.get_same_email().count(), 4)
        self.assertEqual(
            sorted([contact.id for contact in new_contact.get_same_email()]),
            sorted(existing_ids)
        )

        self.assertNotEqual(new_contact.uuid, '')
        self.assertEqual(new_contact.favorite_language, '')
        self.assertTrue(response['Location'].find(reverse('emailing_subscribe_email_done')) >= 0)

        #email verification
        self.assertEqual(len(mail.outbox), 2)

        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [new_contact.email])
        url = reverse('emailing_email_verification', args=[new_contact.uuid])
        email_content = self.email_as_text(verification_email)
        self.assertTrue(email_content.find(url) > 0)

        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])
        email_content = self.email_as_text(notification_email)
        self.assertTrue(email_content.find(new_contact.fullname) > 0)
        self.assertTrue(email_content.find(new_contact.get_absolute_url()) > 0)
        for existing_contact in (existing_contact1, existing_contact2, existing_contact3, existing_contact4):
            self.assertTrue(email_content.find(existing_contact.get_absolute_url()) > 0)

    @override_settings(BALAFON_EMAIL_SUBSCRIBE_ENABLED=False)
    def test_view_subscribe_disabled(self):
        url = reverse("emailing_email_subscribe_newsletter")
        response = self.client.get(url)
        self.assertEqual(404, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 0)

    @override_settings(BALAFON_EMAIL_SUBSCRIBE_ENABLED=False)
    def test_subscribe_disabled(self):
        url = reverse("emailing_email_subscribe_newsletter")

        data = {
            'email': 'pdupond@apidev.fr',
        }

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(404, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 0)

        self.assertEqual(len(mail.outbox), 0)

    @override_settings(BALAFON_DEFAULT_SUBSCRIPTION_TYPE=None)
    def test_email_subscribe_signal(self):
        """test email subscription when no type is configured"""
        url = reverse("emailing_email_subscribe_newsletter")

        def add_to_group_callback(sender, instance, contact, **kwargs):
            group = models.Group.objects.get_or_create(name=instance.subscription_type.name)[0]
            group.contacts.add(contact)
            group.save()

        new_subscription.connect(add_to_group_callback, sender=models.Subscription)

        site1 = Site.objects.get_current()
        site2 = mommy.make(Site)

        subscription_type1 = mommy.make(models.SubscriptionType, name="abc", site=site1)
        subscription_type2 = mommy.make(models.SubscriptionType, name="def", site=site1)
        subscription_type3 = mommy.make(models.SubscriptionType, name="def", site=site2)

        data = {
            'email': 'pdupond@apidev.fr',
        }

        response = self.client.post(url, data=data, follow=False)
        self.assertEqual(302, response.status_code)
        self.assertEqual(models.Contact.objects.count(), 1)

        contact = models.Contact.objects.all()[0]

        self.assertNotEqual(contact.uuid, '')
        self.assertTrue(response['Location'].find(reverse('emailing_subscribe_email_done')) >= 0)

        self.assertEqual(contact.email, data['email'])

        self.assertEqual(len(mail.outbox), 2)

        verification_email = mail.outbox[1]
        self.assertEqual(verification_email.to, [contact.email])
        url = reverse('emailing_email_verification', args=[contact.uuid])
        email_content = self.email_as_text(verification_email)
        self.assertTrue(email_content.find(url) > 0)

        notification_email = mail.outbox[0]
        self.assertEqual(notification_email.to, [settings.BALAFON_NOTIFICATION_EMAIL])
        # Not an Error message
        self.assertTrue(self.email_as_text(notification_email).find("Error") < 0)

        for subscription_type in (subscription_type1, subscription_type2):
            subscription = models.Subscription.objects.get(contact=contact, subscription_type=subscription_type)
            self.assertEqual(subscription.accept_subscription, True)
            self.assertEqual(subscription.subscription_date.date(), date.today())

        for subscription_type in (subscription_type3, ):
            queryset = models.Subscription.objects.filter(contact=contact, subscription_type=subscription_type)
            self.assertEqual(0, queryset.count())

        # Check the callbacks have been called
        self.assertEqual(models.Group.objects.count(), 2)
        for subscription_type in (subscription_type1, subscription_type2, ):
            group = models.Group.objects.get(name=subscription_type.name)
            self.assertEqual(group.entities.count(), 0)
            self.assertEqual(list(group.contacts.all()), [contact])

        new_subscription.disconnect(add_to_group_callback, sender=models.Subscription)




# -*- coding: utf-8 -*-
"""context processor"""

from __future__ import unicode_literals

from balafon.Users.models import CustomMenu


def user_config(request):
    """add constant to context"""

    return {
        'users_custom_menus': [
            menu for menu in CustomMenu.objects.filter(position=CustomMenu.POSITION_MENU) if menu.get_children()
        ],
    }



# -*- coding: utf-8 -*-
"""unit testing"""

from __future__ import unicode_literals

from decimal import Decimal

from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext as _

from coop_cms.tests import BeautifulSoup
from model_mommy import mommy

from balafon.unit_tests import TestCase
from balafon.Crm.models import Action, ActionStatus, ActionType, Contact, Entity
from balafon.Crm.tests import BaseTestCase
from balafon.Store import models

CUSTOM_STYLE = '''<style>
    body {
      background: #000;
    }
</style>'''


class ViewCommercialDocumentTest(TestCase):
    """It should display commercial document"""

    def setUp(self):
        """before each test"""
        super(ViewCommercialDocumentTest, self).setUp()
        user = mommy.make(User, is_active=True, is_staff=True, is_superuser=False)
        user.set_password('abc')
        user.save()
        self.client.login(username=user.username, password='abc')
        self.user = user

    def tearDown(self):
        """after each test"""
        super(ViewCommercialDocumentTest, self).tearDown()
        self.client.logout()

    def test_view_no_sale_action_type(self):
        """It should display standard document"""

        action = mommy.make(Action, type=None)
        mommy.make(models.Sale, action=action)

        url = reverse('store_view_sales_document', args=[action.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context['is_read_only'], False)
        self.assertNotContains(response, CUSTOM_STYLE)

    def test_view_no_custom_template(self):
        """It should display standard document"""

        action_type = mommy.make(ActionType)
        action = mommy.make(Action, type=action_type)
        mommy.make(models.StoreManagementActionType, action_type=action_type, template_name='')

        mommy.make(models.Sale, action=action)

        url = reverse('store_view_sales_document', args=[action.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)

        self.assertNotContains(response, CUSTOM_STYLE)

    def test_view_contacts_and_entities(self):
        """It should display standard document with contacts and entities"""

        action_type = mommy.make(ActionType)
        action = mommy.make(Action, type=action_type)
        mommy.make(models.StoreManagementActionType, action_type=action_type, template_name='')

        contact1 = mommy.make(Contact, lastname='abc' * 10)
        contact2 = mommy.make(Contact, lastname='def' * 10)
        contact3 = mommy.make(Contact, lastname='ghi' * 10)
        entity1 = mommy.make(Entity, name='jkl' * 10)

        action.contacts.add(contact1)
        action.contacts.add(contact2)
        action.entities.add(entity1)
        action.save()

        url = reverse('store_view_sales_document', args=[action.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)

        self.assertContains(response, contact1.fullname)
        self.assertContains(response, contact2.fullname)
        self.assertContains(response, entity1.name)
        self.assertNotContains(response, contact3.fullname)

    def test_view_custom_template(self):
        """It should display custom document"""

        action_type = mommy.make(ActionType)
        action = mommy.make(Action, type=action_type)
        mommy.make(models.StoreManagementActionType, action_type=action_type, template_name='Store/tests/bill.html')

        mommy.make(models.Sale, action=action)

        url = reverse('store_view_sales_document', args=[action.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)

        self.assertContains(response, CUSTOM_STYLE)
        self.assertContains(response, 'big head')
        self.assertContains(response, 'big foot')

    def test_view_read_only(self):
        """It should display as read only"""

        action_type = mommy.make(ActionType)
        action_status = mommy.make(ActionStatus)

        action = mommy.make(Action, type=action_type, status=action_status)

        store_type = mommy.make(models.StoreManagementActionType, action_type=action_type, template_name='')
        store_type.readonly_status.add(action_status)
        store_type.save()

        mommy.make(models.Sale, action=action)

        url = reverse('store_view_sales_document', args=[action.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)

        self.assertEqual(response.context['is_read_only'], True)

    def test_view_frozen_read_only(self):
        """It should display as read only"""

        action_type = mommy.make(ActionType)
        action_status = mommy.make(ActionStatus)

        action = mommy.make(Action, type=action_type, status=action_status, frozen=True)

        store_type = mommy.make(models.StoreManagementActionType, action_type=action_type, template_name='')

        mommy.make(models.Sale, action=action)

        url = reverse('store_view_sales_document', args=[action.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)

        self.assertEqual(response.context['is_read_only'], True)

    def test_view_status_change(self):
        """It should display as writable"""

        action_type = mommy.make(ActionType)
        action_status = mommy.make(ActionStatus)

        action = mommy.make(Action, type=action_type, status=action_status)

        store_type = mommy.make(models.StoreManagementActionType, action_type=action_type, template_name='')

        mommy.make(models.Sale, action=action)

        url = reverse('store_view_sales_document', args=[action.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)

        self.assertEqual(response.context['is_read_only'], False)

    def test_view_no_sales(self):
        """It should display page not found"""
        action = mommy.make(Action, type=None)

        url = reverse('store_view_sales_document', args=[action.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 404)

    def test_view_admin_only(self):
        """It should display permission denied"""
        self.user.is_staff = False
        self.user.save()

        store_action_type = mommy.make(models.StoreManagementActionType)
        action = mommy.make(Action, type=store_action_type.action_type)

        url = reverse('store_view_sales_document', args=[action.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 403)

    def test_view_anonymous(self):
        """It should redirect to login page"""
        self.client.logout()

        store_action_type = mommy.make(models.StoreManagementActionType)
        action = mommy.make(Action, type=store_action_type.action_type)

        url = reverse('store_view_sales_document', args=[action.id])
        response = self.client.get(url)

        self.assertEqual(response.status_code, 302)
        login_url = reverse('login')[3:]
        self.assertTrue(response['Location'].find(login_url) > 0)

    def test_view_values(self):
        """It should display item text"""

        store_action_type = mommy.make(models.StoreManagementActionType)
        action = mommy.make(Action, type=store_action_type.action_type)

        item = mommy.make(models.SaleItem, sale=action.sale, text='Promo ETE', quantity=1, pre_tax_price=10)

        url = reverse('store_view_sales_document', args=[action.id])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, item.text)

    def test_view_unpublished_item(self):
        """It should display item text"""

        store_item = mommy.make(models.StoreItem, published=False)
        store_action_type = mommy.make(models.StoreManagementActionType)
        action = mommy.make(Action, type=store_action_type.action_type)

        item = mommy.make(
            models.SaleItem, sale=action.sale, item=store_item, text='Promo ETE', quantity=1, pre_tax_price=10
        )

        url = reverse('store_view_sales_document', args=[action.id])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, item.text)

    def test_view_public(self):
        """It should display item text"""

        action_type = mommy.make(ActionType, generate_uuid=True)
        mommy.make(models.StoreManagementActionType, action_type=action_type)
        action = mommy.make(Action, type=action_type)

        self.assertNotEqual(action.uuid, '')

        item = mommy.make(models.SaleItem, sale=action.sale, text='Promo ETE', quantity=1, pre_tax_price=10)

        self.client.logout()
        url = reverse('store_view_sales_document_public', args=[action.uuid])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, item.text)

    def test_view_public_utf8(self):
        """It should display item text"""

        action_type = mommy.make(ActionType, generate_uuid=True)
        mommy.make(models.StoreManagementActionType, action_type=action_type)
        action = mommy.make(Action, type=action_type)

        self.assertNotEqual(action.uuid, '')

        item = mommy.make(models.SaleItem, sale=action.sale, text='Promo été', quantity=1, pre_tax_price=10)

        self.client.logout()
        url = reverse('store_view_sales_document_public', args=[action.uuid])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertContains(response, item.text)

    def test_view_public_invalid_uuid(self):
        """It should not display item text"""

        action_type = mommy.make(ActionType, generate_uuid=False)
        mommy.make(models.StoreManagementActionType, action_type=action_type)
        action = mommy.make(Action, type=action_type)

        self.assertEqual(action.uuid, '')

        self.client.logout()

        dummy_uuid = 'xxxxxxxxxxxxx'
        url = reverse('store_view_sales_document_public', args=[dummy_uuid])
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

        url = url.replace(dummy_uuid, '')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)

    def test_view_pdf(self):
        """It should display item text"""

        store_action_type = mommy.make(models.StoreManagementActionType)
        action = mommy.make(Action, type=store_action_type.action_type)

        contact1 = mommy.make(Contact, lastname='abc' * 10)
        action.contacts.add(contact1)
        action.save()

        mommy.make(models.SaleItem, sale=action.sale, text='Promo été', quantity=1, pre_tax_price=10)

        url = reverse('store_view_sales_document_pdf', args=[action.id])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

    def test_view_pdf_make_action_readonly(self):
        """It should display item text"""

        status1 = mommy.make(ActionStatus)
        status2 = mommy.make(ActionStatus)
        status3 = mommy.make(ActionStatus)

        store_action_type = mommy.make(models.StoreManagementActionType)
        action = mommy.make(Action, type=store_action_type.action_type, status=status1)
        store_action_type.default_readonly_status = status3
        store_action_type.readonly_status.add(status2, status3)
        store_action_type.save()

        contact1 = mommy.make(Contact, lastname='abc' * 10)
        action.contacts.add(contact1)
        action.save()

        mommy.make(models.SaleItem, sale=action.sale, text='Promo été', quantity=1, pre_tax_price=10)

        url = reverse('store_view_sales_document_pdf', args=[action.id])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        action  = Action.objects.get(id=action.id)
        self.assertEqual(action.status, status3)

    def test_view_pdf_action_already_readonly(self):
        """It should display item text"""

        status1 = mommy.make(ActionStatus)
        status2 = mommy.make(ActionStatus)
        status3 = mommy.make(ActionStatus)

        store_action_type = mommy.make(models.StoreManagementActionType)
        action = mommy.make(Action, type=store_action_type.action_type, status=status2)
        store_action_type.default_readonly_status = status3
        store_action_type.readonly_status.add(status2, status3)
        store_action_type.save()

        contact1 = mommy.make(Contact, lastname='abc' * 10)
        action.contacts.add(contact1)
        action.save()

        mommy.make(models.SaleItem, sale=action.sale, text='Promo été', quantity=1, pre_tax_price=10)

        url = reverse('store_view_sales_document_pdf', args=[action.id])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)
        action  = Action.objects.get(id=action.id)
        self.assertEqual(action.status, status2)

    def test_view_public_pdf(self):
        """It should display item text"""

        action_type = mommy.make(ActionType, generate_uuid=True)
        mommy.make(models.StoreManagementActionType, action_type=action_type)
        action = mommy.make(Action, type=action_type)

        contact1 = mommy.make(Contact, lastname='abc' * 10)
        action.contacts.add(contact1)
        action.save()

        self.assertNotEqual(action.uuid, '')
        self.client.logout()

        mommy.make(models.SaleItem, sale=action.sale, text='Promo été', quantity=1, pre_tax_price=10)

        url = reverse('store_view_sales_document_pdf_public', args=[action.uuid])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

    def test_view_public_pdf_invalid_uuid(self):
        """It should display item text"""

        action_type = mommy.make(ActionType, generate_uuid=False)
        mommy.make(models.StoreManagementActionType, action_type=action_type)
        action = mommy.make(Action, type=action_type)

        contact1 = mommy.make(Contact, lastname='abc' * 10)
        action.contacts.add(contact1)
        action.save()

        self.assertEqual(action.uuid, '')
        self.client.logout()

        dummy_uuid = 'xxxxxxxxxxxxx'
        url = reverse('store_view_sales_document_public', args=[dummy_uuid])

        response = self.client.get(url)
        self.assertEqual(404, response.status_code)

        url = url.replace(dummy_uuid, '')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 404)


class ActionMailtoTest(BaseTestCase):
    """It should generate mailto properly"""

    def test_mail_to(self):
        """it should generate a mailto link"""
        action_type = mommy.make(ActionType, generate_uuid=False)
        action = mommy.make(Action, type=action_type, subject="Hello")
        contact = mommy.make(Contact, email='toto@toto.fr', lastname='Dalton', firstname='Joe')
        action.contacts.add(contact)
        action.save()
        self.assertEqual(
            action.mail_to, reverse('crm_mailto_action', args=[action.id])
        )

    def test_mail_to_entity(self):
        """it should generate a mailto link"""
        action_type = mommy.make(ActionType, generate_uuid=False)
        action = mommy.make(Action, type=action_type, subject="Hello")
        entity = mommy.make(Entity, email='toto@toto.fr')
        action.entities.add(entity)
        action.save()
        self.assertEqual(
            action.mail_to, reverse('crm_mailto_action', args=[action.id])
        )

    def test_mail_to_entity_contact(self):
        """it should generate a mailto link"""
        action_type = mommy.make(ActionType, generate_uuid=False)
        action = mommy.make(Action, type=action_type, subject="Hello")
        entity = mommy.make(Entity, email='toto@toto.fr')
        action.contacts.add(entity.default_contact)
        action.save()
        self.assertEqual(
            action.mail_to, reverse('crm_mailto_action', args=[action.id])
        )

    def test_mail_to_several(self):
        """it should generate a mailto link"""
        action_type = mommy.make(ActionType, generate_uuid=False)
        action = mommy.make(Action, type=action_type, subject="Hello")

        entity = mommy.make(Entity, email='titi@toto.fr', name='')
        action.entities.add(entity)

        contact = mommy.make(Contact, email='toto@toto.fr', lastname='', firstname='')
        action.contacts.add(contact)

        action.save()
        self.assertEqual(
            action.mail_to, reverse('crm_mailto_action', args=[action.id])
        )

    def test_mail_to_twice(self):
        """it should generate a mailto link"""
        action_type = mommy.make(ActionType, generate_uuid=False)
        action = mommy.make(Action, type=action_type, subject="Hello")

        entity = mommy.make(Entity, email='toto@toto.fr', name='')
        action.entities.add(entity)

        contact = mommy.make(Contact, email='toto@toto.fr', lastname='', firstname='')
        action.contacts.add(contact)

        action.save()

        self.assertEqual(
            action.mail_to, reverse('crm_mailto_action', args=[action.id])
        )

    def test_mail_to_uuid(self):
        """it should generate a mailto link"""
        action_type = mommy.make(ActionType, generate_uuid=True, name='Doc')
        mommy.make(models.StoreManagementActionType, action_type=action_type)
        action = mommy.make(Action, type=action_type, subject="Hello")
        contact = mommy.make(Contact, email='toto@toto.fr')
        action.contacts.add(contact)
        action.save()

        body = _("Here is a link to your {0}: {1}{2}").format(
            action_type.name,
            "http://" + Site.objects.get_current().domain,
            reverse('store_view_sales_document_public', args=[action.uuid])
        )

        self.assertEqual(contact.lastname, '')
        self.assertEqual(contact.firstname, '')

        self.assertEqual(
            action.mail_to, reverse('crm_mailto_action', args=[action.id])
        )

    def test_mail_to_no_email(self):
        """it should not generate a mailto link"""
        action_type = mommy.make(ActionType, generate_uuid=False)
        action = mommy.make(Action, type=action_type, subject="Hello")
        contact = mommy.make(Contact, email='')
        action.contacts.add(contact)
        action.save()
        self.assertEqual(
            action.mail_to, ''
        )

    def test_mail_to_no_contact(self):
        """it should not generate a mailto link"""
        action_type = mommy.make(ActionType, generate_uuid=False)
        action = mommy.make(Action, type=action_type, subject="Hello")
        action.save()
        self.assertEqual(
            action.mail_to, ''
        )


class EditSaleActionTest(BaseTestCase):
    """It should display edit action correctly"""

    def test_edit_action_calculated_amount(self):
        """edit with calculated amount"""
        contact = mommy.make(Contact)

        action_type = mommy.make(ActionType, is_amount_calculated=False)
        mommy.make(
            models.StoreManagementActionType,
            action_type=action_type,
            show_amount_as_pre_tax=False
        )
        action_type = ActionType.objects.get(id=action_type.id)
        self.assertEqual(action_type.is_amount_calculated, True)

        action = mommy.make(Action, type=action_type)

        action.contacts.add(contact)
        action.save()

        url = reverse('crm_edit_action', args=[action.id])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)

        fields = soup.select("#id_amount")
        self.assertEqual(len(fields), 1)

        self.assertEqual(fields[0]["disabled"], "disabled")

    def test_create_action_calculated_amount(self):
        """create with calculated amount"""
        action_type = mommy.make(ActionType, is_amount_calculated=False)
        mommy.make(
            models.StoreManagementActionType,
            action_type=action_type,
            show_amount_as_pre_tax=False
        )
        action_type = ActionType.objects.get(id=action_type.id)
        self.assertEqual(action_type.is_amount_calculated, True)

        url = reverse('crm_create_action_of_type', args=[0, 0, action_type.id])
        response = self.client.get(url)
        self.assertEqual(200, response.status_code)

        soup = BeautifulSoup(response.content)
        fields = soup.select("#id_amount")
        self.assertEqual(len(fields), 1)

        self.assertEqual(fields[0]["disabled"], "disabled")


class CloneSaleActionTest(BaseTestCase):
    """clone an action"""

    def test_post_clone_action(self):
        """it should clone action and sale"""
        action_type_1 = mommy.make(ActionType)
        action_type_2 = mommy.make(ActionType)
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.save()

        store_action_type1 = mommy.make(models.StoreManagementActionType, action_type=action_type_1)
        store_action_type2 = mommy.make(models.StoreManagementActionType, action_type=action_type_2)

        contact = mommy.make(Contact)
        entity = mommy.make(Entity)

        action = mommy.make(models.Action, type=action_type_1, done=True)

        self.assertNotEqual(action.sale, None)

        vat_rate1 = mommy.make(models.VatRate, rate="20.0")
        vat_rate2 = mommy.make(models.VatRate, rate="10.0")

        store_item = mommy.make(models.StoreItem)

        sale_item1 = mommy.make(
            models.SaleItem, quantity=1, pre_tax_price="10", vat_rate=vat_rate1,
            sale=action.sale, order_index=1, item=store_item
        )
        sale_item2 = mommy.make(
            models.SaleItem, quantity=2, pre_tax_price="1", vat_rate=vat_rate2,
            sale=action.sale, order_index=2
        )

        action.contacts.add(contact)
        action.entities.add(entity)

        action.save()

        data = {
            'action_type': action_type_2.id
        }

        response = self.client.post(
            reverse('crm_clone_action', args=[action.id]),
            data=data
        )
        self.assertEqual(200, response.status_code)

        self.assertEqual(2, models.Action.objects.count())
        original_action = models.Action.objects.get(type=action_type_1)
        new_action = models.Action.objects.get(type=action_type_2)

        content = response.content.decode('utf-8')
        self.assertEqual(content, 'reload: {0}'.format(reverse('crm_edit_action', args=[new_action.id])))

        self.assertEqual(original_action.subject, new_action.subject)
        self.assertEqual(new_action.parent, original_action)
        self.assertEqual(new_action.contacts.count(), 1)
        self.assertEqual(new_action.entities.count(), 1)
        self.assertEqual(list(original_action.contacts.all()), list(new_action.contacts.all()))
        self.assertEqual(list(original_action.entities.all()), list(new_action.entities.all()))
        self.assertEqual(new_action.done, False)
        self.assertEqual(original_action.done, True)
        self.assertEqual(new_action.planned_date, original_action.planned_date)
        self.assertEqual(new_action.amount, original_action.amount)
        self.assertEqual(new_action.amount, Decimal("12"))
        self.assertEqual(new_action.sale.saleitem_set.count(), 2)

        for original, clone in zip(original_action.sale.saleitem_set.all(), new_action.sale.saleitem_set.all()):
            self.assertEqual(original.quantity, clone.quantity)
            self.assertEqual(original.vat_rate, clone.vat_rate)
            self.assertEqual(original.pre_tax_price, clone.pre_tax_price)
            self.assertEqual(original.text, clone.text)
            self.assertEqual(original.item, clone.item)

        sale_item = new_action.sale.saleitem_set.all()[0]
        sale_item.quantity = 10
        sale_item.save()

        original_action = models.Action.objects.get(type=action_type_1)
        new_action = models.Action.objects.get(type=action_type_2)
        self.assertNotEqual(new_action.amount, original_action.amount)
        self.assertEqual(original_action.amount, Decimal("12"))
        self.assertEqual(new_action.amount, Decimal("102"))

    def test_post_clone_action_no_sale(self):
        """it should clone action and sale"""
        action_type_1 = mommy.make(ActionType)
        action_type_2 = mommy.make(ActionType)
        action_type_1.next_action_types.add(action_type_2)
        action_type_1.save()

        store_action_type1 = mommy.make(models.StoreManagementActionType, action_type=action_type_1)

        contact = mommy.make(Contact)
        entity = mommy.make(Entity)

        action = mommy.make(models.Action, type=action_type_1, done=True)

        self.assertNotEqual(action.sale, None)

        vat_rate1 = mommy.make(models.VatRate, rate="20.0")
        vat_rate2 = mommy.make(models.VatRate, rate="10.0")

        store_item = mommy.make(models.StoreItem)

        sale_item1 = mommy.make(
            models.SaleItem, quantity=1, pre_tax_price="10", vat_rate=vat_rate1,
            sale=action.sale, order_index=1, item=store_item
        )
        sale_item2 = mommy.make(
            models.SaleItem, quantity=2, pre_tax_price="1", vat_rate=vat_rate2,
            sale=action.sale, order_index=2
        )

        action.contacts.add(contact)
        action.entities.add(entity)

        action.save()

        data = {
            'action_type': action_type_2.id
        }

        response = self.client.post(
            reverse('crm_clone_action', args=[action.id]),
            data=data
        )
        self.assertEqual(200, response.status_code)

        self.assertEqual(2, models.Action.objects.count())
        original_action = models.Action.objects.get(type=action_type_1)
        new_action = models.Action.objects.get(type=action_type_2)

        content = response.content.decode('utf-8')
        self.assertEqual(content, 'reload: {0}'.format(reverse('crm_edit_action', args=[new_action.id])))

        self.assertEqual(original_action.subject, new_action.subject)
        self.assertEqual(new_action.parent, original_action)
        self.assertEqual(new_action.contacts.count(), 1)
        self.assertEqual(new_action.entities.count(), 1)
        self.assertEqual(list(original_action.contacts.all()), list(new_action.contacts.all()))
        self.assertEqual(list(original_action.entities.all()), list(new_action.entities.all()))
        self.assertEqual(new_action.done, False)
        self.assertEqual(original_action.done, True)
        self.assertEqual(new_action.planned_date, original_action.planned_date)
        self.assertEqual(new_action.amount, original_action.amount)
        self.assertEqual(new_action.amount, Decimal("12.0"))
        self.assertEqual(0, models.Sale.objects.filter(action=new_action).count())
